(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/hack-session.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var Crypto = Npm.require("crypto");                                    // 17
var Http = Npm.require("http");                                        // 18
var Https = Npm.require("https");                                      // 19
var Future = Npm.require("fibers/future");                             // 20
var Net = Npm.require("net");                                          // 21
var Dgram = Npm.require("dgram");                                      // 22
var Promise = Npm.require("es6-promise").Promise;                      // 23
var Capnp = Npm.require("capnp");                                      // 24
                                                                       //
var EmailRpc = Capnp.importSystem("sandstorm/email.capnp");            // 26
var HackSessionContext = Capnp.importSystem("sandstorm/hack-session.capnp").HackSessionContext;
var Supervisor = Capnp.importSystem("sandstorm/supervisor.capnp").Supervisor;
var SystemPersistent = Capnp.importSystem("sandstorm/supervisor.capnp").SystemPersistent;
var IpRpc = Capnp.importSystem("sandstorm/ip.capnp");                  // 30
var EmailSendPort = EmailRpc.EmailSendPort;                            // 31
var Grain = Capnp.importSystem("sandstorm/grain.capnp");               // 32
                                                                       //
var Url = Npm.require("url");                                          // 34
                                                                       //
ROOT_URL = Url.parse(process.env.ROOT_URL);                            // 36
HOSTNAME = ROOT_URL.hostname;                                          // 37
                                                                       //
SessionContextImpl = (function () {                                    // 39
  function SessionContextImpl(grainId, sessionId, identityId) {        // 40
    babelHelpers.classCallCheck(this, SessionContextImpl);             //
                                                                       //
    this.grainId = grainId;                                            // 41
    this.sessionId = sessionId;                                        // 42
    this.identityId = identityId;                                      // 43
  }                                                                    //
                                                                       //
  SessionContextImpl.prototype.offer = (function () {                  // 39
    function offer(cap, requiredPermissions, descriptor, displayInfo) {
      var _this2 = this;                                               //
                                                                       //
      return inMeteor(function () {                                    // 47
        if (!_this2.identityId) {                                      // 48
          // TODO(soon): allow non logged in users?                    //
          throw new Meteor.Error(400, "Only logged in users can offer capabilities.");
        }                                                              //
                                                                       //
        var castedCap = cap.castAs(SystemPersistent);                  // 53
        var apiTokenOwner = { webkey: null };                          // 54
        var isUiView = descriptor && descriptor.tags && descriptor.tags.length === 1 && descriptor.tags[0] && descriptor.tags[0].id && descriptor.tags[0].id === Grain.UiView.typeId;
        if (isUiView) {                                                // 58
          apiTokenOwner = {                                            // 59
            user: {                                                    // 60
              identityId: _this2.identityId                            // 61
            }                                                          //
          };                                                           //
        }                                                              //
                                                                       //
        // TODO(soon): This will eventually use SystemPersistent.addRequirements when membranes
        // are fully implemented for supervisors.                      //
        // The following fields will be overwritten by PersistentUiView.save(), so no need to
        // pass them in:                                               //
        //lastUsed: new Date(),                                        //
        //title: "", // This will be replaced by the token's title     //
        //denormalizedGrainMetadata: {}, // This will look up the package for the grain referenced.
        var requirement = {                                            // 73
          permissionsHeld: {                                           // 74
            grainId: _this2.grainId,                                   // 75
            identityId: _this2.identityId,                             // 76
            permissions: requiredPermissions                           // 77
          }                                                            //
        };                                                             //
                                                                       //
        if (!checkRequirements([requirement])) {                       // 81
          throw new Meteor.Error(403, "Permissions not satisfied.");   // 82
        }                                                              //
                                                                       //
        var save = castedCap.save(apiTokenOwner);                      // 85
        var sturdyRef = waitPromise(save).sturdyRef;                   // 86
        ApiTokens.update({ _id: hashSturdyRef(sturdyRef) }, { $push: { requirements: requirement } });
        var powerboxView = isUiView ? {                                // 88
          offer: {                                                     // 89
            uiView: {                                                  // 90
              token: sturdyRef.toString(),                             // 91
              tokenId: hashSturdyRef(sturdyRef.toString())             // 92
            }                                                          //
          }                                                            //
        } : {                                                          //
          offer: {                                                     // 96
            url: ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef
          }                                                            //
        };                                                             //
        Sessions.update({ _id: _this2.sessionId }, {                   // 100
          $set: {                                                      // 102
            powerboxView: powerboxView                                 // 103
          }                                                            //
        });                                                            //
      });                                                              //
    }                                                                  //
                                                                       //
    return offer;                                                      //
  })();                                                                //
                                                                       //
  return SessionContextImpl;                                           //
})();                                                                  //
                                                                       //
Meteor.methods({                                                       // 111
  finishPowerboxRequest: function (webkeyUrl, saveLabel, identityId, grainId) {
    check(webkeyUrl, String);                                          // 113
    check(saveLabel, Match.OneOf(undefined, null, String));            // 114
    check(identityId, String);                                         // 115
    check(grainId, String);                                            // 116
                                                                       //
    var userId = Meteor.userId();                                      // 118
    if (!userId || !globalDb.userHasIdentity(userId, identityId)) {    // 119
      throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
    }                                                                  //
                                                                       //
    var parsedWebkey = Url.parse(webkeyUrl.trim());                    // 123
    var hostId = matchWildcardHost(parsedWebkey.host);                 // 124
    if (!hostId || !globalDb.isApiHostId(hostId)) {                    // 125
      throw new Meteor.Error(500, "Hostname does not match this server. External webkeys are not " + "supported (yet)");
    }                                                                  //
                                                                       //
    var token = parsedWebkey.hash;                                     // 130
    if (!token) {                                                      // 131
      throw new Meteor.Error(400, "Invalid webkey. You must pass in the full webkey, " + "including domain name and hash fragment");
    }                                                                  //
                                                                       //
    token = token.slice(1);                                            // 136
    if (globalDb.isTokenSpecificHostId(hostId) && hostId !== globalDb.apiHostIdForToken(token)) {
      // Note: This case is not security-sensitive. The client could easily compute the correct
      //   host ID for this token. But since they've passed one that doesn't match, we assume
      //   something is wrong and stop here.                           //
      throw new Meteor.Error(400, "Invalid webkey: token doesn't match hostname.");
    }                                                                  //
                                                                       //
    var cap = restoreInternal(hashSturdyRef(token), Match.Optional({ webkey: Match.Optional(Match.Any) }), [], new Buffer(token)).cap;
    var castedCap = cap.castAs(SystemPersistent);                      // 147
    var grainOwner = {                                                 // 148
      grainId: grainId,                                                // 149
      introducerIdentity: identityId                                   // 150
    };                                                                 //
    if (saveLabel) {                                                   // 152
      grainOwner.saveLabel = {                                         // 153
        defaultText: saveLabel                                         // 154
      };                                                               //
    }                                                                  //
                                                                       //
    var save = castedCap.save({ grain: grainOwner });                  // 158
    var sturdyRef = waitPromise(save).sturdyRef;                       // 159
                                                                       //
    return sturdyRef.toString();                                       // 161
  },                                                                   //
                                                                       //
  finishPowerboxOffer: function (sessionId) {                          // 164
    check(sessionId, String);                                          // 165
                                                                       //
    Sessions.update({ _id: sessionId }, { $unset: { powerboxView: null } });
  },                                                                   //
                                                                       //
  getViewInfoForApiToken: function (apiTokenId) {                      // 170
    check(apiTokenId, String);                                         // 171
    var token = globalDb.collections.apiTokens.findOne(apiTokenId);    // 172
    if (!token) {                                                      // 173
      throw new Meteor.Error(400, "No such token.");                   // 174
    }                                                                  //
                                                                       //
    var grain = globalDb.collections.grains.findOne(token.grainId);    // 177
    if (!grain) {                                                      // 178
      throw new Meteor.Error(500, "No grain found for token.");        // 179
    }                                                                  //
                                                                       //
    return grain.cachedViewInfo;                                       // 182
  }                                                                    //
});                                                                    //
                                                                       //
HackSessionContextImpl = (function (_SessionContextImpl) {             // 186
  babelHelpers.inherits(HackSessionContextImpl, _SessionContextImpl);  //
                                                                       //
  function HackSessionContextImpl(grainId, sessionId, identityId) {    // 187
    babelHelpers.classCallCheck(this, HackSessionContextImpl);         //
                                                                       //
    _SessionContextImpl.call(this, grainId, sessionId, identityId);    // 188
  }                                                                    //
                                                                       //
  HackSessionContextImpl.prototype._getPublicId = (function () {       // 186
    function _getPublicId() {                                          // 191
      // Get the grain's public ID, assigning a new one if it doesn't yet have one.
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      while (!this.publicId) {                                         // 196
        // We haven't looked up the public ID yet.                     //
        var grain = Grains.findOne(this.grainId, { fields: { publicId: 1 } });
        if (!grain) throw new Error("Grain does not exist.");          // 199
                                                                       //
        if (grain.publicId) {                                          // 201
          this.publicId = grain.publicId;                              // 202
        } else {                                                       //
          // The grain doesn't have a public ID yet. Generate one.     //
          var candidate = generateRandomHostname(20);                  // 205
                                                                       //
          if (Grains.findOne({ publicId: candidate })) {               // 207
            // This should never ever happen.                          //
            console.error("CRITICAL PROBLEM: Public ID collision. " + "CSPRNG is bad or has insufficient entropy.");
            continue;                                                  // 211
          }                                                            //
                                                                       //
          // Carefully perform an update that becomes a no-op if anyone else has assigned a public ID
          // simultaneously.                                           //
          if (Grains.update({ _id: this.grainId, publicId: { $exists: false } }, { $set: { publicId: candidate } }) > 0) {
            // We won the race.                                        //
            this.publicId = candidate;                                 // 219
          }                                                            //
        }                                                              //
      }                                                                //
                                                                       //
      return this.publicId;                                            // 224
    }                                                                  //
                                                                       //
    return _getPublicId;                                               //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype._getAddress = (function () {        // 186
    function _getAddress() {                                           // 227
      // Get the grain's outgoing e-mail address.                      //
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      return this._getPublicId() + "@" + HOSTNAME;                     // 232
    }                                                                  //
                                                                       //
    return _getAddress;                                                //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype._getUserAddress = (function () {    // 186
    function _getUserAddress() {                                       // 235
      // Get the user's e-mail address.                                //
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      var grain = Grains.findOne(this.grainId, { fields: { identityId: 1 } });
                                                                       //
      var identity = globalDb.getIdentity(grain.identityId);           // 242
                                                                       //
      var primaryEmail = _.findWhere(SandstormDb.getVerifiedEmails(identity), { primary: true });
      var email = primaryEmail && primaryEmail.email || identity.unverifiedEmail;
                                                                       //
      var result = {};                                                 // 247
      if (email) {                                                     // 248
        result.address = email;                                        // 249
      }                                                                //
                                                                       //
      if (identity.profile.name) {                                     // 252
        result.name = identity.profile.name;                           // 253
      }                                                                //
                                                                       //
      return result;                                                   // 256
    }                                                                  //
                                                                       //
    return _getUserAddress;                                            //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.send = (function () {               // 186
    function send(email) {                                             // 259
      return hackSendEmail(this, email);                               // 260
    }                                                                  //
                                                                       //
    return send;                                                       //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getPublicId = (function () {        // 186
    function getPublicId() {                                           // 263
      return inMeteor((function () {                                   // 264
        var result = {};                                               // 265
                                                                       //
        result.publicId = this._getPublicId();                         // 267
        result.hostname = HOSTNAME;                                    // 268
        result.autoUrl = ROOT_URL.protocol + "//" + makeWildcardHost(result.publicId);
                                                                       //
        var grain = Grains.findOne(this.grainId, { fields: { userId: 1 } });
        result.isDemoUser = Meteor.users.findOne(grain.userId).expires ? true : false;
                                                                       //
        return result;                                                 // 274
      }).bind(this));                                                  //
    }                                                                  //
                                                                       //
    return getPublicId;                                                //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.httpGet = (function () {            // 186
    function httpGet(url) {                                            // 278
      var _this = this;                                                // 279
      var session = _this;                                             // 280
                                                                       //
      return new Promise(function (resolve, reject) {                  // 282
        var requestMethod = Http.request;                              // 283
        if (url.indexOf("https://") === 0) {                           // 284
          requestMethod = Https.request;                               // 285
        } else if (url.indexOf("http://") !== 0) {                     //
          err = new Error("Protocol not recognized.");                 // 287
          err.nature = "precondition";                                 // 288
          reject(err);                                                 // 289
        }                                                              //
                                                                       //
        req = requestMethod(url, function (resp) {                     // 292
          var buffers = [];                                            // 293
          var err = undefined;                                         // 294
                                                                       //
          switch (Math.floor(resp.statusCode / 100)) {                 // 296
            case 2:                                                    // 297
              // 2xx response -- OK.                                   //
              resp.on("data", function (buf) {                         // 298
                buffers.push(buf);                                     // 299
              });                                                      //
                                                                       //
              resp.on("end", function () {                             // 302
                resolve({                                              // 303
                  content: Buffer.concat(buffers),                     // 304
                  mimeType: resp.headers["content-type"] || null       // 305
                });                                                    //
              });                                                      //
              break;                                                   // 308
            case 3:                                                    // 308
              // 3xx response -- redirect.                             //
              resolve(session.httpGet(resp.headers.location));         // 310
              break;                                                   // 311
            case 4:                                                    // 311
              // 4xx response -- client error.                         //
              err = new Error("Status code " + resp.statusCode + " received in response.");
              err.nature = "precondition";                             // 314
              reject(err);                                             // 315
              break;                                                   // 316
            case 5:                                                    // 317
              // 5xx response -- internal server error.                //
              err = new Error("Status code " + resp.statusCode + " received in response.");
              err.nature = "localBug";                                 // 319
              reject(err);                                             // 320
              break;                                                   // 321
            default:                                                   // 322
              // ???                                                   //
              err = new Error("Invalid status code " + resp.statusCode + " received in response.");
              err.nature = "localBug";                                 // 324
              reject(err);                                             // 325
              break;                                                   // 326
          }                                                            // 326
        });                                                            //
                                                                       //
        req.on("error", function (e) {                                 // 330
          e.nature = "networkFailure";                                 // 331
          reject(e);                                                   // 332
        });                                                            //
                                                                       //
        req.setTimeout(15000, function () {                            // 335
          req.abort();                                                 // 336
          err = new Error("Request timed out.");                       // 337
          err.nature = "localBug";                                     // 338
          err.durability = "overloaded";                               // 339
          reject(err);                                                 // 340
        });                                                            //
                                                                       //
        req.end();                                                     // 343
      });                                                              //
    }                                                                  //
                                                                       //
    return httpGet;                                                    //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getUserAddress = (function () {     // 186
    function getUserAddress() {                                        // 347
      return inMeteor((function () {                                   // 348
        return this._getUserAddress();                                 // 349
      }).bind(this));                                                  //
    }                                                                  //
                                                                       //
    return getUserAddress;                                             //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteGenerateApiToken = (function () {
    function obsoleteGenerateApiToken(petname, userInfo, expires) {    // 353
      throw new Error("generateApiToken() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteGenerateApiToken;                                   //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteListApiTokens = (function () {
    function obsoleteListApiTokens() {                                 // 357
      throw new Error("listApiTokens() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteListApiTokens;                                      //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteRevokeApiToken = (function () {
    function obsoleteRevokeApiToken(tokenId) {                         // 361
      throw new Error("revokeApiToken() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteRevokeApiToken;                                     //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getUiViewForEndpoint = (function () {
    function getUiViewForEndpoint(url) {                               // 365
      var parsedUrl = Url.parse(url);                                  // 366
                                                                       //
      if (parsedUrl.hash) {                                            // 368
        // Assume that anything with a fragment is a webkey            //
        if (parsedUrl.pathname) {                                      // 369
          throw new Error("Webkey urls cannot contain a path.");       // 370
        }                                                              //
                                                                       //
        var token = parsedUrl.hash.slice(1); // Get rid of # which is always the first character
        var hostId = matchWildcardHost(parsedUrl.host);                // 374
        if (hostId && globalDb.isApiHostId(hostId)) {                  // 375
          // Connecting to a local capability.                         //
          return getWrappedUiViewForToken(token);                      // 377
        } else {                                                       //
          // Connecting to a remote server with a bearer token.        //
          // TODO(someday): Negotiate server-to-server Cap'n Proto connection.
          return { view: new ExternalUiView(url, this.grainId, token) };
        }                                                              //
      } else {                                                         //
        return { view: new ExternalUiView(url, this.grainId) };        // 384
      }                                                                //
    }                                                                  //
                                                                       //
    return getUiViewForEndpoint;                                       //
  })();                                                                //
                                                                       //
  return HackSessionContextImpl;                                       //
})(SessionContextImpl);                                                //
                                                                       //
makeHackSessionContext = function (grainId, sessionId, identityId) {   // 389
  return new Capnp.Capability(new HackSessionContextImpl(grainId, sessionId, identityId), HackSessionContext);
};                                                                     //
                                                                       //
var HOSTNAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789";           // 394
                                                                       //
generateRandomHostname = function (length) {                           // 396
  // Generate a random unique name suitable for use in a hostname.     //
                                                                       //
  var digits = [];                                                     // 399
  for (var i = 0; i < length; i++) {                                   // 400
    digits[i] = Random.choice(HOSTNAME_CHARS);                         // 401
  }                                                                    //
                                                                       //
  return digits.join("");                                              // 404
};                                                                     //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=hack-session.js.map
