(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/admin.js                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var ADMIN_TOKEN_EXPIRATION_TIME = 15 * 60 * 1000;                      // 17
var publicAdminSettings = ["google", "github", "ldap", "emailToken", "splashUrl", "signupDialog", "adminAlert", "adminAlertTime", "adminAlertUrl", "termsUrl", "privacyUrl", "appMarketUrl", "appIndexUrl", "appUpdatesEnabled", "serverTitle", "returnAddress", "ldapNameField"];
                                                                       //
DEFAULT_SIGNUP_DIALOG = "You've been invited to join this Sandstorm server!";
                                                                       //
var adminRoute = RouteController.extend({                              // 27
  template: "admin",                                                   // 28
  waitOn: function () {                                                // 29
    var subs = [Meteor.subscribe("admin", this.params._token), Meteor.subscribe("adminServiceConfiguration", this.params._token)];
    if (this.params._token) {                                          // 34
      subs.push(Meteor.subscribe("adminToken", this.params._token));   // 35
    }                                                                  //
                                                                       //
    return subs;                                                       // 38
  },                                                                   //
                                                                       //
  data: function () {                                                  // 41
    var adminToken = AdminToken.findOne();                             // 42
    return {                                                           // 43
      settings: Settings.find(),                                       // 44
      token: this.params._token,                                       // 45
      isUserPermitted: isAdmin() || adminToken && adminToken.tokenIsValid
    };                                                                 //
  },                                                                   //
                                                                       //
  action: function () {                                                // 50
    // Test the WILDCARD_HOST for sanity.                              //
    Tracker.nonreactive(function () {                                  // 52
      if (Session.get("alreadyTestedWildcardHost")) {                  // 53
        return;                                                        // 54
      }                                                                //
                                                                       //
      HTTP.call("GET", "//" + makeWildcardHost("selftest-" + Random.hexString(20)), { timeout: 2000 }, function (error, response) {
        Session.set("alreadyTestedWildcardHost", true);                // 59
        var looksGood = undefined;                                     // 60
        if (error) {                                                   // 61
          looksGood = false;                                           // 62
        } else {                                                       //
          if (response.statusCode === 200) {                           // 64
            looksGood = true;                                          // 65
          } else {                                                     //
            console.log("Surpring status code from self test domain", response.statusCode);
            looksGood = false;                                         // 68
          }                                                            //
        }                                                              //
                                                                       //
        Session.set("wildcardHostWorks", looksGood);                   // 72
      });                                                              //
    });                                                                //
                                                                       //
    var state = this.state;                                            // 76
    Meteor.call("getSmtpUrl", this.params._token, function (error, result) {
      state.set("smtpUrl", result);                                    // 78
    });                                                                //
                                                                       //
    var user = Meteor.user();                                          // 81
    if (user && user.loginIdentities) {                                // 82
      if (this.params._token) {                                        // 83
        if (!user.signupKey || !user.isAdmin) {                        // 84
          Meteor.call("signUpAsAdmin", this.params._token);            // 85
        } else if (user.isAdmin) {                                     //
          // We don't need the token. Redirect to the current route, minus the token parameter.
          Router.go(this.route.getName(), {}, _.pick(this.params, "query", "hash"));
        }                                                              //
      }                                                                //
    }                                                                  //
                                                                       //
    resetResult(state);                                                // 93
    state.set("configurationServiceName", null);                       // 94
    state.set("token", this.params._token);                            // 95
    this.render();                                                     // 96
  }                                                                    //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 100
  this.route("adminSettings", {                                        // 101
    path: "/admin/settings/:_token?",                                  // 102
    controller: adminRoute                                             // 103
  });                                                                  //
  this.route("adminUsers", {                                           // 105
    path: "/admin/users/:_token?",                                     // 106
    controller: adminRoute                                             // 107
  });                                                                  //
  this.route("adminStats", {                                           // 109
    path: "/admin/stats/:_token?",                                     // 110
    controller: adminRoute                                             // 111
  });                                                                  //
  this.route("adminLog", {                                             // 113
    path: "/admin/log/:_token?",                                       // 114
    controller: adminRoute                                             // 115
  });                                                                  //
  this.route("adminInvites", {                                         // 117
    path: "/admin/invites/:_token?",                                   // 118
    controller: adminRoute                                             // 119
  });                                                                  //
  this.route("adminCaps", {                                            // 121
    path: "/admin/capabilities/:_token?",                              // 122
    controller: adminRoute                                             // 123
  });                                                                  //
  this.route("adminAdvanced", {                                        // 125
    path: "/admin/advanced/:_token?",                                  // 126
    controller: adminRoute                                             // 127
  });                                                                  //
  this.route("adminOld", {                                             // 129
    path: "/admin/:_token?",                                           // 130
    action: function () {                                              // 131
      this.redirect("adminSettings", this.params);                     // 132
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
var resetResult = function (state) {                                   // 137
  state = state || Iron.controller().state;                            // 138
  state.set("numSettings", 1);                                         // 139
  state.set("successes", 0);                                           // 140
  state.set("failures", 0);                                            // 141
  state.set("errors", []);                                             // 142
  state.set("fadeAlert", false);                                       // 143
  state.set("successMessage", "Your settings have been saved.");       // 144
  state.set("powerboxOfferUrl", null);                                 // 145
};                                                                     //
                                                                       //
var getToken = function () {                                           // 148
  var state = Iron.controller().state;                                 // 149
  var token = state.get("token");                                      // 150
  if (!token) {                                                        // 151
    return;                                                            // 152
  } else {                                                             //
    return { _token: token };                                          // 154
  }                                                                    //
};                                                                     //
                                                                       //
var handleError = function (err) {                                     // 158
  var _this2 = this;                                                   //
                                                                       //
  Meteor.setTimeout(function () {                                      // 159
    _this2.set("fadeAlert", true);                                     // 160
  }, 3000);                                                            //
                                                                       //
  if (err) {                                                           // 163
    this.set("failures", this.get("failures") + 1);                    // 164
    console.error(err);                                                // 165
    var errors = this.get("errors");                                   // 166
    errors.push(err);                                                  // 167
    this.set("errors", errors);                                        // 168
  } else {                                                             //
    this.set("successes", this.get("successes") + 1);                  // 170
  }                                                                    //
};                                                                     //
                                                                       //
var updateUser = function (options) {                                  // 174
  var state = Iron.controller().state;                                 // 175
  var token = state.get("token");                                      // 176
  resetResult(state);                                                  // 177
  state.set("successMessage", "User has been updated.");               // 178
  var handleErrorBound = handleError.bind(state);                      // 179
  Meteor.call("adminUpdateUser", token, options, handleErrorBound);    // 180
};                                                                     //
                                                                       //
var capitalize = function (str) {                                      // 183
  return str.charAt(0).toUpperCase() + str.slice(1);                   // 184
};                                                                     //
                                                                       //
if (Meteor.isClient) {                                                 // 187
  (function () {                                                       //
    AdminToken = new Mongo.Collection("adminToken"); // see Meteor.publish("adminToken")
    AdminLog = new Meteor.Collection("adminLog");                      // 189
    Meteor.subscribe("publicAdminSettings");                           // 190
                                                                       //
    Template.admin.helpers({                                           // 192
      adminTab: function () {                                          // 193
        return Router.current().route.getName();                       // 194
      },                                                               //
                                                                       //
      success: function () {                                           // 197
        var state = Iron.controller().state;                           // 198
        return state.get("successes") == state.get("numSettings");     // 199
      },                                                               //
                                                                       //
      failure: function () {                                           // 202
        return Iron.controller().state.get("failures");                // 203
      },                                                               //
                                                                       //
      errors: function () {                                            // 206
        return Iron.controller().state.get("errors");                  // 207
      },                                                               //
                                                                       //
      fadeAlert: function () {                                         // 210
        return Iron.controller().state.get("fadeAlert");               // 211
      },                                                               //
                                                                       //
      successMessage: function () {                                    // 214
        return Iron.controller().state.get("successMessage");          // 215
      },                                                               //
                                                                       //
      settingsActive: function () {                                    // 218
        return Router.current().route.getName() == "adminSettings";    // 219
      },                                                               //
                                                                       //
      usersActive: function () {                                       // 222
        return Router.current().route.getName() == "adminUsers";       // 223
      },                                                               //
                                                                       //
      invitesActive: function () {                                     // 226
        return Router.current().route.getName() == "adminInvites";     // 227
      },                                                               //
                                                                       //
      statsActive: function () {                                       // 230
        return Router.current().route.getName() == "adminStats";       // 231
      },                                                               //
                                                                       //
      logActive: function () {                                         // 234
        return Router.current().route.getName() == "adminLog";         // 235
      },                                                               //
                                                                       //
      capsActive: function () {                                        // 238
        return Router.current().route.getName() == "adminCaps";        // 239
      },                                                               //
                                                                       //
      advancedActive: function () {                                    // 242
        return Router.current().route.getName() == "adminAdvanced";    // 243
      },                                                               //
                                                                       //
      wildcardHostSeemsBroken: function () {                           // 246
        if (Session.get("alreadyTestedWildcardHost") && !Session.get("wildcardHostWorks")) {
          return true;                                                 // 249
        }                                                              //
                                                                       //
        return false;                                                  // 252
      },                                                               //
                                                                       //
      getToken: getToken                                               // 255
    });                                                                //
                                                                       //
    Template.adminSettings.events({                                    // 258
      "click .oauth-checkbox": function (event) {                      // 259
        var state = Iron.controller().state;                           // 260
        var serviceName = event.target.getAttribute("data-servicename");
        var config = Package["service-configuration"].ServiceConfiguration.configurations.findOne({ service: serviceName });
                                                                       //
        var setting = Settings.findOne({ _id: serviceName });          // 264
        if (event.target.checked && (!config || setting && setting.automaticallyReset)) {
          state.set("configurationServiceName", serviceName);          // 266
        }                                                              //
      },                                                               //
                                                                       //
      "click .configure-oauth": function (event) {                     // 270
        var state = Iron.controller().state;                           // 271
        state.set("configurationServiceName", event.target.getAttribute("data-servicename"));
      },                                                               //
                                                                       //
      "click .reset-login-tokens": function (event) {                  // 275
        var state = Iron.controller().state;                           // 276
        resetResult(state);                                            // 277
        var handleErrorBound = handleError.bind(state);                // 278
        Meteor.call("clearResumeTokensForService", this.token, event.target.getAttribute("data-servicename"), handleErrorBound);
      },                                                               //
                                                                       //
      "click #admin-settings-send-toggle": function (event) {          // 283
        var state = Iron.controller().state;                           // 284
        state.set("isEmailTestActive", !state.get("isEmailTestActive"));
        return false; // prevent form from submitting                  // 286
      },                                                               //
                                                                       //
      "click #admin-settings-send-test": function (event) {            // 289
        var state = Iron.controller().state;                           // 290
        resetResult(state);                                            // 291
        var handleErrorBound = handleError.bind(state);                // 292
        state.set("successMessage", "Email has been sent.");           // 293
        Meteor.call("testSend", this.token, document.getElementById("smptUrl").value, document.getElementById("email-test-to").value, handleErrorBound);
        return false; // prevent form from submitting                  // 296
      },                                                               //
                                                                       //
      "submit #admin-settings-form": function (event) {                // 299
        var state = Iron.controller().state;                           // 300
        var token = this.token;                                        // 301
        resetResult(state);                                            // 302
        if (globalDb.isFeatureKeyValid()) {                            // 303
          state.set("numSettings", 9);                                 // 304
        } else {                                                       //
          state.set("numSettings", 4);                                 // 306
        }                                                              //
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 309
        if (event.target.emailTokenLogin.checked && !event.target.smtpUrl.value) {
          handleErrorBound(new Meteor.Error(400, "You must configure an SMTP server to use email login."));
          return false;                                                // 313
        }                                                              //
                                                                       //
        Meteor.call("setAccountSetting", token, "google", event.target.googleLogin.checked, handleErrorBound);
        Meteor.call("setAccountSetting", token, "github", event.target.githubLogin.checked, handleErrorBound);
        Meteor.call("setAccountSetting", token, "emailToken", event.target.emailTokenLogin.checked, handleErrorBound);
        Meteor.call("setSetting", token, "smtpUrl", event.target.smtpUrl.value, handleErrorBound);
        if (globalDb.isFeatureKeyValid()) {                            // 320
          Meteor.call("setAccountSetting", token, "ldap", event.target.ldapLogin.checked, handleErrorBound);
          Meteor.call("setSetting", token, "ldapUrl", event.target.ldapUrl.value, handleErrorBound);
          Meteor.call("setSetting", token, "ldapBase", event.target.ldapBase.value, handleErrorBound);
          Meteor.call("setSetting", token, "ldapDnPattern", event.target.ldapDnPattern.value, handleErrorBound);
          Meteor.call("setSetting", token, "ldapNameField", event.target.ldapNameField.value, handleErrorBound);
        }                                                              //
                                                                       //
        return false;                                                  // 328
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminSettings.helpers({                                   // 332
      setDocumentTitle: function () {                                  // 333
        document.title = "Settings · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      googleSetting: function () {                                     // 337
        return Settings.findOne({ _id: "google" });                    // 338
      },                                                               //
                                                                       //
      githubSetting: function () {                                     // 341
        return Settings.findOne({ _id: "github" });                    // 342
      },                                                               //
                                                                       //
      emailTokenEnabled: function () {                                 // 345
        var setting = Settings.findOne({ _id: "emailToken" });         // 346
        if (setting) {                                                 // 347
          return setting.value;                                        // 348
        } else {                                                       //
          return false;                                                // 350
        }                                                              //
      },                                                               //
                                                                       //
      ldapEnabled: function () {                                       // 354
        var setting = Settings.findOne({ _id: "ldap" });               // 355
        if (setting) {                                                 // 356
          return setting.value;                                        // 357
        } else {                                                       //
          return false;                                                // 359
        }                                                              //
      },                                                               //
                                                                       //
      ldapUrl: function () {                                           // 363
        return globalDb.getLdapUrl() || "ldap://localhost:389";        // 364
      },                                                               //
                                                                       //
      ldapBase: function () {                                          // 367
        return globalDb.getLdapBase() || "OU=People,DC=example,DC=com";
      },                                                               //
                                                                       //
      ldapDnPattern: function () {                                     // 371
        return globalDb.getLdapDnPattern() || "uid=$USERNAME,OU=People,DC=example,DC=com";
      },                                                               //
                                                                       //
      ldapNameField: function () {                                     // 375
        return globalDb.getLdapNameField() || "cn";                    // 376
      },                                                               //
                                                                       //
      smtpUrl: function () {                                           // 379
        return Iron.controller().state.get("smtpUrl");                 // 380
      },                                                               //
                                                                       //
      isEmailTestActive: function () {                                 // 383
        return Iron.controller().state.get("isEmailTestActive");       // 384
      },                                                               //
                                                                       //
      getToken: getToken,                                              // 387
      isFeatureKeyValid: function () {                                 // 388
        return globalDb.isFeatureKeyValid();                           // 389
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminUsers.onCreated(function () {                        // 393
      var state = Iron.controller().state;                             // 394
      var token = state.get("token");                                  // 395
      // TODO(perf): Paginate.                                         //
      this.subscribe("allUsers", token);                               // 397
    });                                                                //
                                                                       //
    Template.adminUsers.events({                                       // 400
      "change select.user-class": function (event) {                   // 401
        var value = event.target.selectedOptions[0].value;             // 402
                                                                       //
        if (value == "admin") {                                        // 404
          updateUser({ userId: this._id, signupKey: true, isAdmin: true });
        } else if (value == "invited") {                               //
          updateUser({ userId: this._id, signupKey: true, isAdmin: false });
        } else if (value == "guest") {                                 //
          updateUser({ userId: this._id, signupKey: false, isAdmin: false });
        } else {                                                       //
          console.error("unrecognized user class");                    // 411
        }                                                              //
      },                                                               //
                                                                       //
      "change .is-signedup-checkbox": function (event) {               // 415
        // The userid is stored on the the <tr>, which is always 2 nodes up
        var userId = event.target.parentElement.parentElement.getAttribute("data-userid");
      }                                                                //
    });                                                                //
    Template.adminUsers.helpers({                                      // 420
      setDocumentTitle: function () {                                  // 421
        document.title = "Users · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      users: function () {                                             // 425
        return Meteor.users.find({ loginIdentities: { $exists: 1 } }, { sort: { createdAt: 1 } });
      },                                                               //
                                                                       //
      userIdentity: function () {                                      // 429
        var identityId = SandstormDb.getUserIdentityIds(this)[0];      // 430
        var identity = Meteor.users.findOne({ _id: identityId });      // 431
        if (identity) {                                                // 432
          SandstormDb.fillInProfileDefaults(identity);                 // 433
          SandstormDb.fillInIntrinsicName(identity);                   // 434
          return identity;                                             // 435
        }                                                              //
      },                                                               //
                                                                       //
      userSignupNote: function () {                                    // 439
        if (this.signupEmail) {                                        // 440
          return this.signupEmail;                                     // 441
        } else if (this.signupNote) {                                  //
          return this.signupNote;                                      // 443
        } else {                                                       //
          return "";                                                   // 445
        }                                                              //
      },                                                               //
                                                                       //
      userIsAdmin: function () {                                       // 449
        return !!this.isAdmin;                                         // 450
      },                                                               //
                                                                       //
      userIsInvited: function () {                                     // 453
        return !this.isAdmin && !!this.signupKey;                      // 454
      },                                                               //
                                                                       //
      userIsGuest: function () {                                       // 457
        return !this.isAdmin && !this.signupKey;                       // 458
      },                                                               //
                                                                       //
      userStorageUsage: function () {                                  // 461
        return typeof this.storageUsage === "number" ? prettySize(this.storageUsage) : "";
      }                                                                //
    });                                                                //
                                                                       //
    var configureLoginServiceDialogTemplateForService = function (serviceName) {
      return Template["configureLoginServiceDialogFor" + capitalize(serviceName)];
    };                                                                 //
                                                                       //
    var configurationFields = function (serviceName) {                 // 470
      var template = configureLoginServiceDialogTemplateForService(serviceName);
      return template ? template.fields() : [];                        // 472
    };                                                                 //
                                                                       //
    Template._adminConfigureLoginServiceDialog.helpers({               // 475
      configurationFields: function () {                               // 476
        var serviceName = Iron.controller().state.get("configurationServiceName");
        var configurations = Package["service-configuration"].ServiceConfiguration.configurations;
        var configuration = configurations.findOne({ service: serviceName });
        var fields = configurationFields(serviceName);                 // 480
        if (configuration) {                                           // 481
          return _.map(fields, function (field) {                      // 482
            field.value = configuration[field.property];               // 483
            return field;                                              // 484
          });                                                          //
        } else {                                                       //
          return fields;                                               // 487
        }                                                              //
      },                                                               //
                                                                       //
      visible: function () {                                           // 491
        return Iron.controller().state.get("configurationServiceName") !== null;
      },                                                               //
                                                                       //
      configurationSteps: function () {                                // 495
        // renders the appropriate template                            //
        return configureLoginServiceDialogTemplateForService(Iron.controller().state.get("configurationServiceName"));
      }                                                                //
    });                                                                //
                                                                       //
    Template._adminConfigureLoginServiceDialog.events({                // 502
      "click .configure-login-service-dismiss-button": function () {   // 503
        Iron.controller().state.set("configurationServiceName", null);
      },                                                               //
                                                                       //
      "click #configure-login-service-dialog-save-configuration": function () {
        var state = Iron.controller().state;                           // 508
        resetResult(state);                                            // 509
                                                                       //
        // This is a bit of a hack, but we set the number high so that a success message is never displayed
        state.set("numSettings", 100);                                 // 512
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 514
        var serviceName = state.get("configurationServiceName");       // 515
        var token = this.token;                                        // 516
        var configuration = {                                          // 517
          service: serviceName                                         // 518
        };                                                             //
                                                                       //
        // Fetch the value of each input field                         //
        _.each(configurationFields(serviceName), function (field) {    // 522
          configuration[field.property] = document.getElementById("configure-login-service-dialog-" + field.property).value.trim();
        });                                                            //
                                                                       //
        configuration.loginStyle = "redirect";                         // 527
                                                                       //
        Meteor.call("adminConfigureLoginService", token, configuration, function (err) {
          handleErrorBound(err);                                       // 530
          state.set("configurationServiceName", null);                 // 531
        });                                                            //
                                                                       //
        Meteor.call("setAccountSetting", token, serviceName, true, handleErrorBound);
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminInvites.events({                                     // 538
      "click #send": function (event) {                                // 539
        var state = Iron.controller().state;                           // 540
        var from = document.getElementById("invite-from").value;       // 541
        var list = document.getElementById("invite-emails").value;     // 542
        var subject = document.getElementById("invite-subject").value;
        var message = document.getElementById("invite-message").value;
        var quotaInput = document.getElementById("invite-quota");      // 545
        var quota = undefined;                                         // 546
        if (quotaInput && quotaInput.value.trim() !== "") {            // 547
          quota = parseInt(quotaInput.value);                          // 548
        }                                                              //
                                                                       //
        var sendButton = event.currentTarget;                          // 551
        sendButton.disabled = true;                                    // 552
        var oldContent = sendButton.textContent;                       // 553
        sendButton.textContent = "Sending...";                         // 554
                                                                       //
        Meteor.call("sendInvites", state.get("token"), getOrigin(), from, list, subject, message, quota, function (error, results) {
          sendButton.disabled = false;                                 // 558
          sendButton.textContent = oldContent;                         // 559
          if (error) {                                                 // 560
            state.set("inviteMessage", { error: error.toString() });   // 561
          } else {                                                     //
            state.set("inviteMessage", results);                       // 563
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #create": function (event) {                              // 568
        var state = Iron.controller().state;                           // 569
        var note = document.getElementById("key-note").value;          // 570
        var quotaInput = document.getElementById("key-quota");         // 571
        var quota = undefined;                                         // 572
        if (quotaInput && quotaInput.value.trim() !== "") {            // 573
          quota = parseInt(quotaInput.value);                          // 574
        }                                                              //
                                                                       //
        Meteor.call("createSignupKey", state.get("token"), note, quota, function (error, key) {
          if (error) {                                                 // 578
            state.set("inviteMessage", { error: error.toString() });   // 579
          } else {                                                     //
            state.set("inviteMessage", {                               // 581
              url: getOrigin() + Router.routes.signup.path({ key: key })
            });                                                        //
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #set-quota-submit": function (event) {                    // 588
        var state = Iron.controller().state;                           // 589
        var list = document.getElementById("set-quota-emails").value;  // 590
        var quotaInput = document.getElementById("set-quota-quota");   // 591
        var quota = undefined;                                         // 592
        if (quotaInput && quotaInput.value.trim() !== "") {            // 593
          quota = parseInt(quotaInput.value);                          // 594
        }                                                              //
                                                                       //
        var updateButton = event.currentTarget;                        // 597
        updateButton.disabled = true;                                  // 598
        var oldContent = updateButton.textContent;                     // 599
        updateButton.textContent = "Updating...";                      // 600
                                                                       //
        Meteor.call("updateQuotas", state.get("token"), list, quota, function (error, results) {
          updateButton.disabled = false;                               // 603
          updateButton.textContent = oldContent;                       // 604
          if (error) {                                                 // 605
            state.set("inviteMessage", { error: error.toString() });   // 606
          } else {                                                     //
            document.getElementById("set-quota-emails").value = "";    // 608
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click .autoSelect": function (event) {                          // 613
        event.currentTarget.select();                                  // 614
      },                                                               //
                                                                       //
      "click #retry": function (event) {                               // 617
        Iron.controller().state.set("inviteMessage", undefined);       // 618
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminInvites.helpers({                                    // 622
      setDocumentTitle: function () {                                  // 623
        document.title = "Invites · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      error: function () {                                             // 627
        var res = Iron.controller().state.get("inviteMessage");        // 628
        return res && res.error;                                       // 629
      },                                                               //
                                                                       //
      email: function () {                                             // 632
        var me = Meteor.user();                                        // 633
        var email = me.services && me.services.google && me.services.google.email || me.services && me.services.github && me.services.github.email;
        if (email && me.profile.name) {                                // 636
          email = me.profile.name + " <" + email + ">";                // 637
        }                                                              //
                                                                       //
        email = email || "";                                           // 640
        return email;                                                  // 641
      },                                                               //
                                                                       //
      url: function () {                                               // 644
        var res = Iron.controller().state.get("inviteMessage");        // 645
        return res && res.url;                                         // 646
      },                                                               //
                                                                       //
      sent: function () {                                              // 649
        var res = Iron.controller().state.get("inviteMessage");        // 650
        return res && res.sent;                                        // 651
      }                                                                //
    });                                                                //
                                                                       //
    var maybeScrollLog = function () {                                 // 655
      // This function appears unused.  TODO: verify safe to delete    //
      var elem = document.getElementById("adminLog");                  // 657
      var scrollLogToBottom = function (elem) {                        // 658
        elem.scrollTop = elem.scrollHeight;                            // 659
      };                                                               //
                                                                       //
      if (elem) {                                                      // 662
        // The log already exists. It's about to be updated. Check if it's scrolled to the bottom
        // before the update.                                          //
        if (elem.scrollHeight - elem.scrollTop === elem.clientHeight) {
          // Indeed, so we want to scroll it back to the bottom after the update.
          Tracker.afterFlush(function () {                             // 667
            scrollLogToBottom(elem);                                   // 667
          });                                                          //
        }                                                              //
      } else {                                                         //
        // No element exists yet, but it's probably about to be created, in which case we definitely
        // want to scroll it.                                          //
        Tracker.afterFlush(function () {                               // 672
          var elem2 = document.getElementById("adminLog");             // 673
          if (elem2) scrollLogToBottom(elem2);                         // 674
        });                                                            //
      }                                                                //
    };                                                                 //
                                                                       //
    Template.adminLog.onCreated(function () {                          // 679
      var state = Iron.controller().state;                             // 680
      var token = state.get("token");                                  // 681
      this.subscribe("adminLog", token);                               // 682
    });                                                                //
                                                                       //
    Template.adminLog.helpers({                                        // 685
      setDocumentTitle: function () {                                  // 686
        document.title = "Log · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      html: function () {                                              // 690
        // jscs:disable requireCamelCaseOrUpperCaseIdentifiers         //
        return AnsiUp.ansi_to_html(AdminLog.find({}, { $sort: { _id: 1 } }).map(function (entry) {
          return entry.text;                                           // 693
        }).join(""), { use_classes: true });                           //
        // jscs:enable requireCamelCaseOrUpperCaseIdentifiers          //
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminCaps.onCreated(function () {                         // 699
      var state = Iron.controller().state;                             // 700
      var token = state.get("token");                                  // 701
      this.subscribe("adminApiTokens", token);                         // 702
                                                                       //
      // TODO(perf): Don't subscribe to all users, just to the ones that are referenced from
      //   the relevant ApiTokens.                                     //
      this.subscribe("allUsers", token);                               // 706
    });                                                                //
                                                                       //
    Template.adminCaps.helpers({                                       // 709
      setDocumentTitle: function () {                                  // 710
        document.title = "Capabilities · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      showPowerboxOffer: function () {                                 // 714
        var state = Iron.controller().state;                           // 715
        return !!state.get("powerboxOfferUrl");                        // 716
      },                                                               //
                                                                       //
      powerboxOfferData: function () {                                 // 719
        var state = Iron.controller().state;                           // 720
        return {                                                       // 721
          get: function () {                                           // 722
            return {                                                   // 723
              offer: {                                                 // 724
                url: state.get("powerboxOfferUrl")                     // 725
              },                                                       //
              onDismiss: function () {                                 // 727
                state.set("powerboxOfferUrl", null);                   // 728
              }                                                        //
            };                                                         //
          }                                                            //
        };                                                             //
      },                                                               //
                                                                       //
      caps: function () {                                              // 735
        return ApiTokens.find({                                        // 736
          $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
        });                                                            //
      },                                                               //
                                                                       //
      userName: function () {                                          // 744
        var userId = findAdminUserForToken(this);                      // 745
        var user = Meteor.users.findOne({ _id: userId });              // 746
        if (!user) {                                                   // 747
          return "no user";                                            // 748
        }                                                              //
                                                                       //
        var identityId = SandstormDb.getUserIdentityIds(user)[0];      // 751
        var identity = Meteor.users.findOne({ _id: identityId });      // 752
        if (identity) {                                                // 753
          SandstormDb.fillInProfileDefaults(identity);                 // 754
          return identity.profile.name;                                // 755
        }                                                              //
      },                                                               //
                                                                       //
      isDisabled: function () {                                        // 759
        return !this.userId;                                           // 760
      },                                                               //
                                                                       //
      disabled: function () {                                          // 763
        return this.revoked;                                           // 764
      }                                                                //
    });                                                                //
                                                                       //
    var updateCap = function (capId, value) {                          // 768
      var state = Iron.controller().state;                             // 769
      resetResult(state);                                              // 770
      if (!value) {                                                    // 771
        state.set("successMessage", "Capability has been re-enabled.");
      } else {                                                         //
        state.set("successMessage", "Capability has been disabled.");  // 774
      }                                                                //
                                                                       //
      var handleErrorBound = handleError.bind(state);                  // 777
      Meteor.call("adminToggleDisableCap", state.get("token"), capId, value, handleErrorBound);
    };                                                                 //
                                                                       //
    Template.adminCaps.events({                                        // 781
      "click #offer-ipnetwork": function (event) {                     // 782
        var state = Iron.controller().state;                           // 783
        resetResult(state);                                            // 784
        state.set("successMessage", "IpNetwork webkey created. Look for it in the top bar.");
        Meteor.call("offerIpNetwork", this.token, function (err, webkey) {
          state.set("powerboxOfferUrl", webkey);                       // 787
          handleError.call(state, err);                                // 788
        });                                                            //
                                                                       //
        return false; // prevent form from submitting                  // 791
      },                                                               //
                                                                       //
      "click #offer-ipinterface": function (event) {                   // 794
        var state = Iron.controller().state;                           // 795
        resetResult(state);                                            // 796
        state.set("successMessage", "IpInterface webkey created. Look for it in the top bar.");
        Meteor.call("offerIpInterface", this.token, function (err, webkey) {
          state.set("powerboxOfferUrl", webkey);                       // 799
          handleError.call(state, err);                                // 800
        });                                                            //
                                                                       //
        return false; // prevent form from submitting                  // 803
      },                                                               //
                                                                       //
      "click #powerbox-offer-popup-closer": function (event) {         // 806
        var state = Iron.controller().state;                           // 807
        return state.set("powerboxOfferUrl", null);                    // 808
      },                                                               //
                                                                       //
      "click .disable-cap": function (event) {                         // 811
        var capId = event.target.getAttribute("data-id");              // 812
        var token = ApiTokens.findOne({ _id: capId });                 // 813
                                                                       //
        updateCap(capId, !token.revoked);                              // 815
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminAdvanced.events({                                    // 819
      "submit #admin-settings-form": function (event) {                // 820
        var state = Iron.controller().state;                           // 821
        var token = this.token;                                        // 822
        resetResult(state);                                            // 823
        state.set("numSettings", 12);                                  // 824
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 826
        Meteor.call("setSetting", token, "serverTitle", event.target.serverTitle.value, handleErrorBound);
        Meteor.call("setSetting", token, "returnAddress", event.target.returnAddress.value, handleErrorBound);
        Meteor.call("setSetting", token, "splashUrl", event.target.splashUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "signupDialog", event.target.signupDialog.value, handleErrorBound);
        Meteor.call("setSetting", token, "termsUrl", event.target.termsUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "privacyUrl", event.target.privacyUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "adminAlert", event.target.adminAlert.value, handleErrorBound);
        Meteor.call("setSetting", token, "appMarketUrl", event.target.appMarketUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "appIndexUrl", event.target.appIndexUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "appUpdatesEnabled", event.target.appUpdatesEnabled.checked, handleErrorBound);
        var alertTimeString = event.target.alertTime.value.trim();     // 839
        if (alertTimeString) {                                         // 840
          var alertTime = new Date(alertTimeString);                   // 841
          if (isNaN(alertTime.getTime())) {                            // 842
            // Assume only time and not date was set.                  //
            alertTime = new Date(new Date().toLocaleDateString() + " " + alertTimeString);
          }                                                            //
                                                                       //
          if (isNaN(alertTime.getTime())) {                            // 847
            handleErrorBound(new Meteor.Error(400, "Couldn't parse alert time, please be more precise."));
          } else {                                                     //
            Meteor.call("setSetting", token, "adminAlertTime", alertTime, handleErrorBound);
          }                                                            //
        } else {                                                       //
          Meteor.call("setSetting", token, "adminAlertTime", null, handleErrorBound);
        }                                                              //
                                                                       //
        Meteor.call("setSetting", token, "adminAlertUrl", event.target.alertUrl.value, handleErrorBound);
        return false;                                                  // 858
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminAdvanced.helpers({                                   // 862
      setDocumentTitle: function () {                                  // 863
        document.title = "Advanced · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      serverTitle: function () {                                       // 867
        var setting = Settings.findOne({ _id: "serverTitle" });        // 868
        return setting && setting.value || "";                         // 869
      },                                                               //
                                                                       //
      returnAddress: function () {                                     // 872
        var setting = Settings.findOne({ _id: "returnAddress" });      // 873
        return setting && setting.value || "";                         // 874
      },                                                               //
                                                                       //
      splashUrl: function () {                                         // 877
        var setting = Settings.findOne({ _id: "splashUrl" });          // 878
        return setting && setting.value || "";                         // 879
      },                                                               //
                                                                       //
      origin: function () {                                            // 882
        return getOrigin();                                            // 882
      },                                                               //
                                                                       //
      signupDialog: function () {                                      // 884
        var setting = Settings.findOne({ _id: "signupDialog" });       // 885
        return setting && setting.value || DEFAULT_SIGNUP_DIALOG;      // 886
      },                                                               //
                                                                       //
      termsUrl: function () {                                          // 889
        var setting = Settings.findOne({ _id: "termsUrl" });           // 890
        return setting && setting.value;                               // 891
      },                                                               //
                                                                       //
      privacyUrl: function () {                                        // 894
        var setting = Settings.findOne({ _id: "privacyUrl" });         // 895
        return setting && setting.value;                               // 896
      },                                                               //
                                                                       //
      adminAlert: function () {                                        // 899
        var setting = Settings.findOne({ _id: "adminAlert" });         // 900
        return setting && setting.value;                               // 901
      },                                                               //
                                                                       //
      alertTime: function () {                                         // 904
        var setting = Settings.findOne({ _id: "adminAlertTime" });     // 905
        if (setting && setting.value) {                                // 906
          return setting.value.toLocaleDateString() + " " + setting.value.toLocaleTimeString();
        } else {                                                       //
          return "";                                                   // 909
        }                                                              //
      },                                                               //
                                                                       //
      alertUrl: function () {                                          // 913
        var setting = Settings.findOne({ _id: "adminAlertUrl" });      // 914
        return setting && setting.value;                               // 915
      },                                                               //
                                                                       //
      appMarketUrl: function () {                                      // 918
        var setting = Settings.findOne({ _id: "appMarketUrl" });       // 919
        return setting && setting.value;                               // 920
      },                                                               //
                                                                       //
      appIndexUrl: function () {                                       // 923
        var setting = Settings.findOne({ _id: "appIndexUrl" });        // 924
        return setting && setting.value;                               // 925
      },                                                               //
                                                                       //
      appUpdatesEnabled: function () {                                 // 928
        var setting = Settings.findOne({ _id: "appUpdatesEnabled" });  // 929
        return setting && setting.value;                               // 930
      }                                                                //
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
if (Meteor.isServer) {                                                 // 935
  (function () {                                                       //
    var Fs = Npm.require("fs");                                        // 936
    var SANDSTORM_ADMIN_TOKEN = SANDSTORM_VARDIR + "/adminToken";      // 937
                                                                       //
    var getSmtpUrl = function () {                                     // 939
      var setting = Settings.findOne({ _id: "smtpUrl" });              // 940
      if (setting) {                                                   // 941
        return setting.value;                                          // 942
      } else {                                                         //
        return process.env.MAIL_URL;                                   // 944
      }                                                                //
    };                                                                 //
                                                                       //
    var tokenIsValid = function (token) {                              // 948
      if (token && Fs.existsSync(SANDSTORM_ADMIN_TOKEN)) {             // 949
        var stats = Fs.statSync(SANDSTORM_ADMIN_TOKEN);                // 950
        var expireTime = new Date(Date.now() - ADMIN_TOKEN_EXPIRATION_TIME);
        if (stats.mtime < expireTime) {                                // 952
          return false;                                                // 953
        } else {                                                       //
          return Fs.readFileSync(SANDSTORM_ADMIN_TOKEN, { encoding: "utf8" }) === token;
        }                                                              //
      } else {                                                         //
        return false;                                                  // 958
      }                                                                //
    };                                                                 //
                                                                       //
    var checkAuth = function (token) {                                 // 962
      check(token, Match.OneOf(undefined, null, String));              // 963
      if (!isAdmin() && !tokenIsValid(token)) {                        // 964
        throw new Meteor.Error(403, "User must be admin or provide a valid token");
      }                                                                //
    };                                                                 //
                                                                       //
    var clearAdminToken = function (token) {                           // 969
      if (tokenIsValid(token)) {                                       // 970
        Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                          // 971
        console.log("Admin token deleted.");                           // 972
      }                                                                //
    };                                                                 //
                                                                       //
    Meteor.methods({                                                   // 976
      setAccountSetting: function (token, serviceName, value) {        // 977
        checkAuth(token);                                              // 978
        check(serviceName, String);                                    // 979
        check(value, Boolean);                                         // 980
                                                                       //
        // TODO(someday): currently this relies on the fact that an account is tied to a single
        // identity, and thus has only that entry in "services". This will need to be looked at when
        // multiple login methods/identities are allowed for a single account.
        if (!value && !tokenIsValid(token) && serviceName in Meteor.user().services) {
          throw new Meteor.Error(403, "You can not disable the login service that your account uses.");
        }                                                              //
                                                                       //
        // Only check configurations for OAuth services.               //
        var oauthServices = ["google", "github"];                      // 991
        if (value && oauthServices.indexOf(serviceName) != -1) {       // 992
          var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
          var config = ServiceConfiguration.configurations.findOne({ service: serviceName });
          if (!config) {                                               // 995
            throw new Meteor.Error(403, "You must configure the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
          }                                                            //
                                                                       //
          if (!config.clientId || !config.secret) {                    // 1000
            throw new Meteor.Error(403, "You must provide a non-empty clientId and secret for the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
          }                                                            //
        }                                                              //
                                                                       //
        Settings.upsert({ _id: serviceName }, { $set: { value: value } });
        if (value) {                                                   // 1007
          Settings.update({ _id: serviceName }, { $unset: { automaticallyReset: 1 } });
        }                                                              //
      },                                                               //
                                                                       //
      setSetting: function (token, name, value) {                      // 1012
        checkAuth(token);                                              // 1013
        check(name, String);                                           // 1014
        check(value, Match.OneOf(null, String, Date, Boolean));        // 1015
                                                                       //
        Settings.upsert({ _id: name }, { $set: { value: value } });    // 1017
        if (name.lastIndexOf("ldap", 0) === 0) {                       // 1018
          // some ldap settings need to be set for the accounts-ldap package
          var ldapSetting = name.slice(4);                             // 1020
          if (ldapSetting === "Url" || ldapSetting === "Base") {       // 1021
            LDAP_DEFAULTS[ldapSetting.toLowerCase()] = value;          // 1022
          }                                                            //
        }                                                              //
      },                                                               //
                                                                       //
      getSmtpUrl: function (token) {                                   // 1027
        checkAuth(token);                                              // 1028
                                                                       //
        return getSmtpUrl();                                           // 1030
      },                                                               //
                                                                       //
      adminConfigureLoginService: function (token, options) {          // 1033
        checkAuth(token);                                              // 1034
        check(options, Match.ObjectIncluding({ service: String }));    // 1035
                                                                       //
        var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
                                                                       //
        ServiceConfiguration.configurations.upsert({ service: options.service }, options);
      },                                                               //
                                                                       //
      clearResumeTokensForService: function (token, serviceName) {     // 1042
        checkAuth(token);                                              // 1043
        check(serviceName, String);                                    // 1044
                                                                       //
        var query = {};                                                // 1046
        query["services." + serviceName] = { $exists: true };          // 1047
        Meteor.users.find(query).forEach(function (identity) {         // 1048
          if (identity.services.resume && identity.services.resume.loginTokens && identity.services.resume.loginTokens.length > 0) {
            Meteor.users.update({ _id: identity._id }, { $set: { "services.resume.loginTokens": [] } });
          }                                                            //
                                                                       //
          Meteor.users.update({ "loginIdentities.id": identity._id }, { $set: { "services.resume.loginTokens": [] } });
        });                                                            //
      },                                                               //
                                                                       //
      adminUpdateUser: function (token, userInfo) {                    // 1059
        checkAuth(token);                                              // 1060
        check(userInfo, {                                              // 1061
          userId: String,                                              // 1062
          signupKey: Boolean,                                          // 1063
          isAdmin: Boolean                                             // 1064
        });                                                            //
                                                                       //
        var userId = userInfo.userId;                                  // 1067
        if (userId === Meteor.userId() && !userInfo.isAdmin) {         // 1068
          throw new Meteor.Error(403, "User cannot remove admin permissions from itself.");
        }                                                              //
                                                                       //
        Meteor.users.update({ _id: userId }, { $set: _.omit(userInfo, ["_id", "userId"]) });
      },                                                               //
                                                                       //
      testSend: function (token, smtpUrl, to) {                        // 1075
        checkAuth(token);                                              // 1076
        check(smtpUrl, String);                                        // 1077
        check(to, String);                                             // 1078
                                                                       //
        SandstormEmail.send({                                          // 1080
          to: to,                                                      // 1081
          from: globalDb.getServerTitle() + " <" + globalDb.getReturnAddress() + ">",
          subject: "Testing your Sandstorm's SMTP setting",            // 1083
          text: "Success! Your outgoing SMTP is working.",             // 1084
          smtpUrl: smtpUrl                                             // 1085
        });                                                            //
      },                                                               //
                                                                       //
      createSignupKey: function (token, note, quota) {                 // 1089
        checkAuth(token);                                              // 1090
        check(note, String);                                           // 1091
        check(quota, Match.OneOf(undefined, null, Number));            // 1092
                                                                       //
        var key = Random.id();                                         // 1094
        var content = { _id: key, used: false, note: note };           // 1095
        if (typeof quota === "number") content.quota = quota;          // 1096
        SignupKeys.insert(content);                                    // 1097
        return key;                                                    // 1098
      },                                                               //
                                                                       //
      sendInvites: function (token, origin, from, list, subject, message, quota) {
        checkAuth(token);                                              // 1102
        check([origin, from, list, subject, message], [String]);       // 1103
        check(quota, Match.OneOf(undefined, null, Number));            // 1104
                                                                       //
        if (!from.trim()) {                                            // 1106
          throw new Meteor.Error(403, "Must enter 'from' address.");   // 1107
        }                                                              //
                                                                       //
        if (!list.trim()) {                                            // 1110
          throw new Meteor.Error(403, "Must enter 'to' addresses.");   // 1111
        }                                                              //
                                                                       //
        this.unblock();                                                // 1114
                                                                       //
        list = list.split("\n");                                       // 1116
        for (var i in babelHelpers.sanitizeForInObject(list)) {        // 1117
          var email = list[i].trim();                                  // 1118
                                                                       //
          if (email) {                                                 // 1120
            var key = Random.id();                                     // 1121
                                                                       //
            var content = {                                            // 1123
              _id: key,                                                // 1124
              used: false,                                             // 1125
              note: "E-mail invite to " + email,                       // 1126
              email: email,                                            // 1127
              definitelySent: false                                    // 1128
            };                                                         //
            if (typeof quota === "number") content.quota = quota;      // 1130
            SignupKeys.insert(content);                                // 1131
            SandstormEmail.send({                                      // 1132
              to: email,                                               // 1133
              from: from,                                              // 1134
              subject: subject,                                        // 1135
              text: message.replace(/\$KEY/g, origin + Router.routes.signup.path({ key: key }))
            });                                                        //
            SignupKeys.update(key, { $set: { definitelySent: true } });
          }                                                            //
        }                                                              //
                                                                       //
        return { sent: true };                                         // 1142
      },                                                               //
                                                                       //
      offerIpNetwork: function (token) {                               // 1145
        checkAuth(token);                                              // 1146
        if (!isAdmin()) {                                              // 1147
          throw new Meteor.Error(403, "Offering IpNetwork is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
        }                                                              //
                                                                       //
        var requirements = [{ userIsAdmin: Meteor.userId() }];         // 1152
        var sturdyRef = waitPromise(saveFrontendRef({ ipNetwork: true }, { webkey: null }, requirements)).sturdyRef;
        return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
      },                                                               //
                                                                       //
      offerIpInterface: function (token) {                             // 1161
        checkAuth(token);                                              // 1162
        if (!isAdmin()) {                                              // 1163
          throw new Meteor.Error(403, "Offering IpInterface is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
        }                                                              //
                                                                       //
        var requirements = [{ userIsAdmin: Meteor.userId() }];         // 1168
        var sturdyRef = waitPromise(saveFrontendRef({ ipInterface: true }, { webkey: null }, requirements)).sturdyRef;
        return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
      },                                                               //
                                                                       //
      adminToggleDisableCap: function (token, capId, value) {          // 1177
        checkAuth(token);                                              // 1178
        check(capId, String);                                          // 1179
        check(value, Boolean);                                         // 1180
                                                                       //
        if (value) {                                                   // 1182
          ApiTokens.update({ _id: capId }, { $set: { revoked: true } });
        } else {                                                       //
          ApiTokens.update({ _id: capId }, { $set: { revoked: false } });
        }                                                              //
      },                                                               //
                                                                       //
      updateQuotas: function (token, list, quota) {                    // 1189
        checkAuth(token);                                              // 1190
        check(list, String);                                           // 1191
        check(quota, Match.OneOf(undefined, null, Number));            // 1192
                                                                       //
        if (!list.trim()) {                                            // 1194
          throw new Meteor.Error(400, "Must enter addresses.");        // 1195
        }                                                              //
                                                                       //
        var items = list.split("\n");                                  // 1198
        var invalid = [];                                              // 1199
        for (var i in babelHelpers.sanitizeForInObject(items)) {       // 1200
          var modifier = typeof quota === "number" ? { $set: { quota: quota } } : { $unset: { quota: "" } };
          var n = SignupKeys.update({ email: items[i] }, modifier, { multi: true });
          n += Meteor.users.update({ signupEmail: items[i] }, modifier, { multi: true });
                                                                       //
          if (n < 1) invalid.push(items[i]);                           // 1206
        }                                                              //
                                                                       //
        if (invalid.length > 0) {                                      // 1209
          throw new Meteor.Error(404, "These addresses did not map to any user nor invite: " + invalid.join(", "));
        }                                                              //
      },                                                               //
                                                                       //
      dismissAdminStatsNotifications: function (token) {               // 1215
        checkAuth(token);                                              // 1216
        globalDb.collections.notifications.remove({ "admin.type": "reportStats" });
      },                                                               //
                                                                       //
      signUpAsAdmin: function (token) {                                // 1220
        check(token, String);                                          // 1221
        checkAuth(token);                                              // 1222
        if (!this.userId) {                                            // 1223
          throw new Meteor.Error(403, "Must be logged in to sign up as admin.");
        }                                                              //
                                                                       //
        if (!Meteor.user().loginIdentities) {                          // 1227
          throw new Meteor.Error(403, "Must be logged into an account to sign up as admin.");
        }                                                              //
                                                                       //
        Meteor.users.update({ _id: this.userId }, { $set: { isAdmin: true, signupKey: "admin" } });
        clearAdminToken(token);                                        // 1232
      }                                                                //
    });                                                                //
                                                                       //
    var authorizedAsAdmin = function (token, userId) {                 // 1236
      return Match.test(token, Match.OneOf(undefined, null, String)) && (userId && isAdminById(userId) || tokenIsValid(token));
    };                                                                 //
                                                                       //
    Meteor.publish("admin", function (token) {                         // 1241
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1242
      return Settings.find();                                          // 1243
    });                                                                //
                                                                       //
    Meteor.publish("adminServiceConfiguration", function (token) {     // 1246
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1247
      return Package["service-configuration"].ServiceConfiguration.configurations.find();
    });                                                                //
                                                                       //
    Meteor.publish("publicAdminSettings", function () {                // 1251
      return Settings.find({ _id: { $in: publicAdminSettings } });     // 1252
    });                                                                //
                                                                       //
    Meteor.publish("adminToken", function (token) {                    // 1255
      check(token, String);                                            // 1256
      this.added("adminToken", "adminToken", { tokenIsValid: tokenIsValid(token) });
      this.ready();                                                    // 1258
    });                                                                //
                                                                       //
    Meteor.publish("allUsers", function (token) {                      // 1261
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1262
      return Meteor.users.find();                                      // 1263
    });                                                                //
                                                                       //
    Meteor.publish("activityStats", function (token) {                 // 1266
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1267
      return ActivityStats.find();                                     // 1268
    });                                                                //
                                                                       //
    Meteor.publish("statsTokens", function (token) {                   // 1271
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1272
      return StatsTokens.find();                                       // 1273
    });                                                                //
                                                                       //
    Meteor.publish("allPackages", function (token) {                   // 1276
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1277
      return Packages.find({ manifest: { $exists: true } }, { fields: { appId: 1, "manifest.appVersion": 1,
          "manifest.actions": 1, "manifest.appTitle": 1 } });          // 1280
    });                                                                //
                                                                       //
    Meteor.publish("realTimeStats", function (token) {                 // 1283
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1284
                                                                       //
      // Last five minutes.                                            //
      this.added("realTimeStats", "now", computeStats(new Date(Date.now() - 5 * 60 * 1000)));
                                                                       //
      // Since last sample.                                            //
      var lastSample = ActivityStats.findOne({}, { sort: { timestamp: -1 } });
      var lastSampleTime = lastSample ? lastSample.timestamp : new Date(0);
      this.added("realTimeStats", "today", computeStats(lastSampleTime));
                                                                       //
      // TODO(someday): Update every few minutes?                      //
                                                                       //
      this.ready();                                                    // 1296
    });                                                                //
                                                                       //
    Meteor.publish("adminLog", function (token) {                      // 1299
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1300
                                                                       //
      var logfile = SANDSTORM_LOGDIR + "/sandstorm.log";               // 1302
                                                                       //
      var fd = Fs.openSync(logfile, "r");                              // 1304
      var startSize = Fs.fstatSync(fd).size;                           // 1305
                                                                       //
      // Start tailing at EOF - 8k.                                    //
      var offset = Math.max(0, startSize - 8192);                      // 1308
                                                                       //
      var _this = this;                                                // 1310
      function doTail() {                                              // 1311
        for (;;) {                                                     // 1312
          var buf = new Buffer(Math.max(1024, startSize - offset));    // 1313
          var n = Fs.readSync(fd, buf, 0, buf.length, offset);         // 1314
          if (n <= 0) break;                                           // 1315
          _this.added("adminLog", offset, { text: buf.toString("utf8", 0, n) });
          offset += n;                                                 // 1317
        }                                                              //
      }                                                                //
                                                                       //
      // Watch the file for changes.                                   //
      var watcher = Fs.watch(logfile, { persistent: false }, Meteor.bindEnvironment(doTail));
                                                                       //
      // When the subscription stops, stop watching the file.          //
      this.onStop(function () {                                        // 1325
        watcher.close();                                               // 1326
        Fs.closeSync(fd);                                              // 1327
      });                                                              //
                                                                       //
      // Read initial 8k tail data immediately.                        //
      doTail();                                                        // 1331
                                                                       //
      // Notify ready.                                                 //
      this.ready();                                                    // 1334
    });                                                                //
                                                                       //
    Meteor.publish("adminApiTokens", function (token) {                // 1337
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1338
      return ApiTokens.find({                                          // 1339
        $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
      }, {                                                             //
        fields: {                                                      // 1345
          frontendRef: 1,                                              // 1346
          created: 1,                                                  // 1347
          requirements: 1,                                             // 1348
          revoked: 1                                                   // 1349
        }                                                              //
      });                                                              //
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
function serviceEnabled(name) {                                        // 1355
  var setting = Settings.findOne({ _id: name });                       // 1356
  return setting && !!setting.value;                                   // 1357
}                                                                      //
                                                                       //
if (Meteor.isServer) {                                                 // 1360
  function observeOauthService(name) {                                 // 1361
    Settings.find({ _id: name, value: true }).observe({                // 1362
      added: function () {                                             // 1363
        // Tell the oauth library it should accept login attempts from this service.
        Accounts.oauth.registerService(name);                          // 1365
      },                                                               //
                                                                       //
      removed: function () {                                           // 1368
        // Tell the oauth library it should deny login attempts from this service.
        Accounts.oauth.unregisterService(name);                        // 1370
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  observeOauthService("github");                                       // 1375
  observeOauthService("google");                                       // 1376
}                                                                      //
                                                                       //
Accounts.identityServices.github = {                                   // 1379
  isEnabled: function () {                                             // 1380
    return serviceEnabled("github");                                   // 1381
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1384
    name: "oauthLoginButton",                                          // 1385
    priority: 1,                                                       // 1386
    data: {                                                            // 1387
      method: "loginWithGithub",                                       // 1388
      name: "github",                                                  // 1389
      displayName: "GitHub"                                            // 1390
    }                                                                  //
  }                                                                    //
};                                                                     //
                                                                       //
Accounts.identityServices.google = {                                   // 1395
  isEnabled: function () {                                             // 1396
    return serviceEnabled("google");                                   // 1397
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1400
    name: "oauthLoginButton",                                          // 1401
    priority: 2,                                                       // 1402
    data: {                                                            // 1403
      method: "loginWithGoogle",                                       // 1404
      name: "google",                                                  // 1405
      displayName: "Google"                                            // 1406
    }                                                                  //
  }                                                                    //
};                                                                     //
                                                                       //
Accounts.identityServices.email = {                                    // 1411
  isEnabled: function () {                                             // 1412
    return serviceEnabled("emailToken");                               // 1413
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1416
    name: "emailLoginForm",                                            // 1417
    priority: 10                                                       // 1418
  }                                                                    //
};                                                                     //
                                                                       //
Accounts.identityServices.ldap = {                                     // 1422
  isEnabled: function () {                                             // 1423
    return serviceEnabled("ldap") && globalDb.isFeatureKeyValid();     // 1424
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1427
    name: "ldapLoginForm",                                             // 1428
    priority: 20 }                                                     // 1429
};                                                                     //
// Put it at the bottom of the list.                                   //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=admin.js.map
