(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/grain.js                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
// This file implements /grain, i.e. the main view into an app.        //
                                                                       //
var DEFAULT_TITLE = "Sandstorm";                                       // 19
                                                                       //
if (Meteor.isServer) {                                                 // 21
  (function () {                                                       //
    var Crypto = Npm.require("crypto");                                // 22
                                                                       //
    Meteor.publish("grainTopBar", function (grainId) {                 // 24
      check(grainId, String);                                          // 25
                                                                       //
      var result = [Grains.find({                                      // 27
        _id: grainId,                                                  // 29
        $or: [{ userId: this.userId }, { "private": { $ne: true } }]   // 30
      }, {                                                             //
        fields: {                                                      // 35
          title: 1,                                                    // 36
          userId: 1,                                                   // 37
          identityId: 1,                                               // 38
          "private": 1                                                 // 39
        }                                                              //
      })];                                                             //
      if (this.userId) {                                               // 43
        var myIdentityIds = SandstormDb.getUserIdentityIds(globalDb.getUser(this.userId));
        result.push(ApiTokens.find({                                   // 45
          grainId: grainId,                                            // 46
          $or: [{ "owner.user.identityId": { $in: myIdentityIds } }, { identityId: { $in: myIdentityIds } }]
        }));                                                           //
      }                                                                //
                                                                       //
      return result;                                                   // 54
    });                                                                //
                                                                       //
    // We allow users to learn package information about a grain they own.
    // This is used for obtaining icon and app title information for grains
    // you own, which is used in the sidebar. It is not a security/privacy
    // risk since it only exposes this information for grains the user owns.
    Meteor.publish("packageByGrainId", function (grainId) {            // 61
      check(grainId, String);                                          // 62
      var publishThis = [];                                            // 63
      // We need to publish the packageId so that client-side code can
      // find the right package.                                       //
      var thisGrainCursor = Grains.find({                              // 66
        _id: grainId,                                                  // 67
        userId: this.userId                                            // 68
      }, {                                                             //
        fields: { packageId: 1 }                                       // 70
      });                                                              //
      publishThis.push(thisGrainCursor);                               // 72
                                                                       //
      if (thisGrainCursor.count()) {                                   // 74
        var thisGrain = thisGrainCursor.fetch()[0];                    // 75
        var thisPackageCursor = Packages.find({ _id: thisGrain.packageId });
        publishThis.push(thisPackageCursor);                           // 77
      }                                                                //
                                                                       //
      return publishThis;                                              // 80
    });                                                                //
                                                                       //
    Meteor.publish("tokenInfo", function (token) {                     // 83
      // Allows the client side to map a raw token to its entry in ApiTokens, and the additional
      // metadata that it will need to display the app icon and title.  We do not care about making
      // the metadata reactive.                                        //
      check(token, String);                                            // 87
                                                                       //
      var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");
      var apiToken = ApiTokens.findOne({                               // 90
        _id: hashedToken                                               // 91
      }, {                                                             //
        fields: {                                                      // 93
          grainId: 1,                                                  // 94
          owner: 1                                                     // 95
        }                                                              //
      });                                                              //
      if (!apiToken) {                                                 // 98
        this.added("tokenInfo", token, { invalidToken: true });        // 99
      } else {                                                         //
        var grainId = apiToken.grainId;                                // 101
        var grain = Grains.findOne({                                   // 102
          _id: grainId                                                 // 103
        }, {                                                           //
          fields: {                                                    // 105
            packageId: 1,                                              // 106
            appId: 1                                                   // 107
          }                                                            //
        });                                                            //
        if (!grain) {                                                  // 110
          this.added("tokenInfo", token, { invalidToken: true });      // 111
        } else {                                                       //
          if (apiToken.owner && apiToken.owner.user) {                 // 113
            var identity = globalDb.getIdentity(apiToken.owner.user.identityId);
            var metadata = apiToken.owner.user.denormalizedGrainMetadata;
            if (identity && metadata) {                                // 116
              this.added("tokenInfo", token, { identityOwner: _.pick(identity, "_id", "profile"),
                grainId: grainId,                                      // 119
                grainMetadata: metadata });                            // 120
            } else {                                                   //
              this.added("tokenInfo", token, { invalidToken: true });  // 122
            }                                                          //
          } else if (!apiToken.owner || "webkey" in apiToken.owner) {  //
            var pkg = Packages.findOne({ _id: grain.packageId }, { fields: { manifest: 1 } });
            var appTitle = pkg && pkg.manifest && pkg.manifest.appTitle || { defaultText: "" };
            var appIcon = undefined;                                   // 127
            if (pkg && pkg.manifest && pkg.manifest.metadata && pkg.manifest.metadata.icons) {
              var icons = pkg.manifest.metadata.icons;                 // 129
              appIcon = icons.grain || icons.appGrid;                  // 130
            }                                                          //
                                                                       //
            var denormalizedGrainMetadata = {                          // 133
              appTitle: appTitle,                                      // 134
              icon: appIcon,                                           // 135
              appId: appIcon ? undefined : grain.appId                 // 136
            };                                                         //
            this.added("tokenInfo", token, { webkey: true,             // 138
              grainId: grainId,                                        // 140
              grainMetadata: denormalizedGrainMetadata });             // 141
          } else {                                                     //
            this.added("tokenInfo", token, { invalidToken: true });    // 143
          }                                                            //
        }                                                              //
      }                                                                //
                                                                       //
      this.ready();                                                    // 148
      return;                                                          // 149
    });                                                                //
                                                                       //
    Meteor.publish("requestingAccess", function (grainId) {            // 152
      check(grainId, String);                                          // 153
                                                                       //
      if (!this.userId) {                                              // 155
        throw new Meteor.Error(403, "Must be logged in to request access.");
      }                                                                //
                                                                       //
      var identityIds = SandstormDb.getUserIdentityIds(Meteor.users.findOne({ _id: this.userId }));
                                                                       //
      var grain = globalDb.getGrain(grainId);                          // 161
      if (!grain) {                                                    // 162
        throw new Meteor.Error(404, "Grain not found.");               // 163
      }                                                                //
                                                                       //
      if (grain.userId === this.userId) {                              // 166
        this.added("grantedAccessRequests", Random.id(), { grainId: grainId, identityId: grain.identityId });
      }                                                                //
                                                                       //
      var _this = this;                                                // 171
      var query = ApiTokens.find({ grainId: grainId, accountId: grain.userId,
        "owner.user.identityId": { $in: identityIds },                 // 173
        revoked: { $ne: true } });                                     // 174
      var handle = query.observe({                                     // 175
        added: function (apiToken) {                                   // 176
          _this.added("grantedAccessRequests", Random.id(), { grainId: grainId, identityId: apiToken.owner.user.identityId });
        }                                                              //
      });                                                              //
                                                                       //
      this.onStop(function () {                                        // 182
        return handle.stop();                                          //
      });                                                              //
    });                                                                //
                                                                       //
    Meteor.publish("grainSize", function (grainId) {                   // 185
      // Publish pseudo-collection containing the size of the grain opened in the given session.
      check(grainId, String);                                          // 187
                                                                       //
      var grain = Grains.findOne(grainId);                             // 189
      if (!grain || grain.userId !== this.userId) {                    // 190
        return [];                                                     // 191
      }                                                                //
                                                                       //
      var supervisor = globalBackend.cap().getGrain(this.userId, grainId).supervisor;
                                                                       //
      var _this = this;                                                // 196
      var stopped = false;                                             // 197
      var promise = getGrainSize(supervisor);                          // 198
                                                                       //
      function getNext(oldSize) {                                      // 200
        promise = getGrainSize(supervisor, oldSize);                   // 201
        promise.then(function (size) {                                 // 202
          if (!stopped) {                                              // 203
            if (size !== oldSize) {                                    // 204
              // sometimes there are false alarms                      //
              _this.changed("grainSizes", grainId, { size: size });    // 205
            }                                                          //
                                                                       //
            getNext(size);                                             // 208
          }                                                            //
        }, function (err) {                                            //
                                                                       //
          if (!stopped) {                                              // 212
            if (err.kjType === "disconnected") {                       // 213
              _this.stop();                                            // 214
            } else {                                                   //
              _this.error(err);                                        // 216
            }                                                          //
          }                                                            //
        });                                                            //
      }                                                                //
                                                                       //
      promise.then(function (size) {                                   // 222
        if (!stopped) {                                                // 223
          _this.added("grainSizes", grainId, { size: size });          // 224
          _this.ready();                                               // 225
          getNext(size);                                               // 226
        }                                                              //
      }, function (err) {                                              //
                                                                       //
        if (!stopped) {                                                // 230
          if (err.kjType === "disconnected") {                         // 231
            _this.stop();                                              // 232
          } else {                                                     //
            _this.error(err);                                          // 234
          }                                                            //
        }                                                              //
      });                                                              //
                                                                       //
      _this.onStop(function () {                                       // 239
        stopped = true;                                                // 240
        promise.cancel();                                              // 241
      });                                                              //
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
// GrainSizes is used by grainview.js                                  //
GrainSizes = new Mongo.Collection("grainSizes");                       // 247
// TokenInfo is used by grainview.js                                   //
TokenInfo = new Mongo.Collection("tokenInfo");                         // 249
GrantedAccessRequests = new Mongo.Collection("grantedAccessRequests");
// Pseudo-collections published above.                                 //
                                                                       //
function emailLinkWithInlineStyle(url, text) {                         // 253
  return "<a href='" + url + "' style='display:inline-block;text-decoration:none;" + "font-family:sans-serif;width:200px;min-height:30px;line-height:30px;" + "border-radius:4px;text-align:center;background:#428bca;color:white'>" + text + "</a>";
}                                                                      //
                                                                       //
Meteor.methods({                                                       // 260
  deleteGrain: function (grainId) {                                    // 261
    check(grainId, String);                                            // 262
                                                                       //
    if (this.userId) {                                                 // 264
      var grain = Grains.findOne({ _id: grainId, userId: this.userId });
      if (grain) {                                                     // 266
        Grains.remove(grainId);                                        // 267
        globalDb.removeApiTokens({                                     // 268
          grainId: grainId,                                            // 269
          $or: [{ owner: { $exists: false } }, { owner: { webkey: null } }]
        });                                                            //
        if (grain.lastUsed) {                                          // 275
          DeleteStats.insert({ type: isDemoUser() ? "demoGrain" : "grain",
            lastActive: grain.lastUsed, appId: grain.appId });         // 277
        }                                                              //
                                                                       //
        if (!this.isSimulation) {                                      // 280
          waitPromise(globalBackend.deleteGrain(grainId, this.userId));
          Meteor.call("deleteUnusedPackages", grain.appId);            // 282
        }                                                              //
      }                                                                //
    }                                                                  //
  },                                                                   //
                                                                       //
  forgetGrain: function (grainId, identityId) {                        // 288
    // TODO(cleanup): For now we are ignoring `identityId`, but maybe we should expose finer-grained
    //  forgetting.                                                    //
                                                                       //
    check(grainId, String);                                            // 292
    check(identityId, String);                                         // 293
                                                                       //
    if (!this.userId) {                                                // 295
      throw new Meteor.Error(403, "Must be logged in to forget a grain.");
    }                                                                  //
                                                                       //
    SandstormDb.getUserIdentityIds(Meteor.user()).forEach(function (identityId) {
      globalDb.removeApiTokens({ grainId: grainId, "owner.user.identityId": identityId });
    });                                                                //
  },                                                                   //
                                                                       //
  updateGrainPreferredIdentity: function (grainId, identityId) {       // 304
    check(grainId, String);                                            // 305
    check(identityId, String);                                         // 306
    if (!this.userId) {                                                // 307
      throw new Meteor.Error(403, "Must be logged in.");               // 308
    }                                                                  //
                                                                       //
    var grain = globalDb.getGrain(grainId) || {};                      // 311
    if (!grain.userId === this.userId) {                               // 312
      throw new Meteor.Error(403, "Grain not owned by current user.");
    }                                                                  //
                                                                       //
    Grains.update({ _id: grainId }, { $set: { identityId: identityId } });
  },                                                                   //
                                                                       //
  updateGrainTitle: function (grainId, newTitle, identityId) {         // 319
    check(grainId, String);                                            // 320
    check(newTitle, String);                                           // 321
    check(identityId, String);                                         // 322
    if (this.userId) {                                                 // 323
      var grain = Grains.findOne(grainId);                             // 324
      if (grain) {                                                     // 325
        if (grain.userId === this.userId) {                            // 326
          Grains.update({ _id: grainId, userId: this.userId }, { $set: { title: newTitle } });
        } else {                                                       //
          if (!globalDb.userHasIdentity(this.userId, identityId)) {    // 329
            throw new Meteor.Error(403, "Current user does not have identity " + identityId);
          }                                                            //
                                                                       //
          var token = ApiTokens.findOne({                              // 333
            grainId: grainId,                                          // 334
            objectId: { $exists: false },                              // 335
            "owner.user.identityId": identityId                        // 336
          }, {                                                         //
            sort: { created: 1 }                                       // 338
          });                                                          //
          if (token) {                                                 // 340
            ApiTokens.update(token._id, { $set: { "owner.user.title": newTitle } });
          }                                                            //
        }                                                              //
      }                                                                //
    }                                                                  //
  },                                                                   //
                                                                       //
  privatizeGrain: function (grainId) {                                 // 348
    check(grainId, String);                                            // 349
    if (this.userId) {                                                 // 350
      Grains.update({ _id: grainId, userId: this.userId }, { $set: { "private": true } });
    }                                                                  //
  },                                                                   //
                                                                       //
  inviteUsersToGrain: function (origin, identityId, grainId, title, roleAssignment, contacts, message) {
    var _this2 = this;                                                 //
                                                                       //
    if (!this.isSimulation) {                                          // 357
      var _ret2 = (function () {                                       //
        check(origin, String);                                         // 358
        check(identityId, String);                                     // 359
        check(grainId, String);                                        // 360
        check(title, String);                                          // 361
        check(roleAssignment, roleAssignmentPattern);                  // 362
        check(contacts, [{                                             // 363
          _id: String,                                                 // 365
          isDefault: Match.Optional(Boolean),                          // 366
          profile: Match.ObjectIncluding({                             // 367
            service: String,                                           // 368
            name: String,                                              // 369
            intrinsicName: String                                      // 370
          })                                                           //
        }]);                                                           //
        check(message, { text: String, html: String });                // 374
        if (!_this2.userId) {                                          // 375
          throw new Meteor.Error(403, "Must be logged in to share by email.");
        }                                                              //
                                                                       //
        if (!globalDb.userHasIdentity(_this2.userId, identityId)) {    // 379
          throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
        }                                                              //
                                                                       //
        if (contacts.length === 0) {                                   // 383
          throw new Meteor.Error(400, "No contacts were provided.");   // 384
        }                                                              //
                                                                       //
        var accountId = _this2.userId;                                 // 387
        var identity = globalDb.getIdentity(identityId);               // 388
        var sharerDisplayName = identity.profile.name;                 // 389
        var outerResult = { successes: [], failures: [] };             // 390
        contacts.forEach(function (contact) {                          // 391
          if (contact.isDefault && contact.profile.service === "email") {
            var emailAddress = contact.profile.intrinsicName;          // 393
            var result = SandstormPermissions.createNewApiToken(globalDb, { identityId: identityId, accountId: accountId }, grainId, "email invitation for " + emailAddress, roleAssignment, { webkey: { forSharing: true } });
            var url = origin + "/shared/" + result.token;              // 398
            var html = message.html + "<br><br>" + emailLinkWithInlineStyle(url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>" + "Note: If you forward this email to other people, they will be able to access " + "the share as well. To prevent this, remove the button before forwarding.</div>";
            try {                                                      // 404
              SandstormEmail.send({                                    // 405
                to: emailAddress,                                      // 406
                from: "Sandstorm server <no-reply@" + HOSTNAME + ">",  // 407
                subject: sharerDisplayName + " has invited you to join a grain: " + title,
                text: message.text + "\n\nFollow this link to open the shared grain:\n\n" + url + "\n\nNote: If you forward this email to other people, they will be able to " + "access the share as well. To prevent this, remove the link before forwarding.",
                html: html                                             // 412
              });                                                      //
            } catch (e) {                                              //
              outerResult.failures.push({ contact: contact, error: e.toString() });
            }                                                          //
          } else {                                                     //
            var result = SandstormPermissions.createNewApiToken(globalDb, { identityId: identityId, accountId: accountId }, grainId, "direct invitation to " + contact.profile.intrinsicName, roleAssignment, { user: { identityId: contact._id, title: title } });
            var url = origin + "/shared/" + result.token;              // 422
            try {                                                      // 423
              var _identity = Meteor.users.findOne({ _id: contact._id });
              var email = _.findWhere(SandstormDb.getVerifiedEmails(_identity), { primary: true });
              if (email) {                                             // 427
                var intrinsicName = contact.profile.intrinsicName;     // 428
                var loginNote = undefined;                             // 429
                if (contact.profile.service === "google") {            // 430
                  loginNote = "Google account with address " + email.email;
                } else if (contact.profile.service === "github") {     //
                  loginNote = "Github account with username " + intrinsicName;
                } else if (contact.profile.service === "email") {      //
                  loginNote = "email address " + intrinsicName;        // 435
                } else {                                               //
                  throw new Meteor.Error(500, "Unknown service to email share link.");
                }                                                      //
                                                                       //
                var html = message.html + "<br><br>" + emailLinkWithInlineStyle(url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>" + "Note: You will need to log in with your " + loginNote + " to access this grain.";
                SandstormEmail.send({                                  // 445
                  to: email.email,                                     // 446
                  from: "Sandstorm server <no-reply@" + HOSTNAME + ">",
                  subject: sharerDisplayName + " has invited you to join a grain: " + title,
                  text: message.text + "\n\nFollow this link to open the shared grain:\n\n" + url + "\n\nNote: You will need to log in with your " + loginNote + " to access this grain.",
                  html: html                                           // 452
                });                                                    //
              } else {                                                 //
                outerResult.failures.push({ contact: contact, warning: "User does not have a " + "verified email, so notification of this share was not sent to them. Please " + "manually share " + url + " with them." });
              }                                                        //
            } catch (e) {                                              //
              outerResult.failures.push({ contact: contact, error: e.toString(),
                warning: "Share succeeded, but there was an error emailing the user. Please " + "manually share " + url + " with them." });
            }                                                          //
          }                                                            //
        });                                                            //
                                                                       //
        return {                                                       // 467
          v: outerResult                                               //
        };                                                             //
      })();                                                            //
                                                                       //
      if (typeof _ret2 === "object") return _ret2.v;                   //
    }                                                                  //
  },                                                                   //
                                                                       //
  requestAccess: function (origin, grainId, identityId) {              // 471
    check(origin, String);                                             // 472
    check(grainId, String);                                            // 473
    check(identityId, String);                                         // 474
    if (!this.isSimulation) {                                          // 475
      if (!this.userId) {                                              // 476
        throw new Meteor.Error(403, "Must be logged in to request access.");
      }                                                                //
                                                                       //
      if (!globalDb.userHasIdentity(this.userId, identityId)) {        // 480
        throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
      }                                                                //
                                                                       //
      var grain = Grains.findOne(grainId);                             // 484
      if (!grain) {                                                    // 485
        throw new Meteor.Error(404, "No such grain");                  // 486
      }                                                                //
                                                                       //
      var grainOwner = globalDb.getUser(grain.userId);                 // 489
      var email = _.findWhere(SandstormDb.getUserEmails(grainOwner), { primary: true });
      if (!email) {                                                    // 491
        throw new Meteor.Error("no email", "Grain owner has no email address.");
      }                                                                //
                                                                       //
      var emailAddress = email.email;                                  // 495
                                                                       //
      var identity = globalDb.getIdentity(identityId);                 // 497
      globalDb.addContact(grainOwner._id, identityId);                 // 498
                                                                       //
      var envelopeFrom = globalDb.getReturnAddress();                  // 500
      var fromEmail = globalDb.getServerTitle() + " <" + globalDb.getReturnAddress() + ">";
      var senderEmails = SandstormDb.getVerifiedEmails(identity);      // 502
      var senderPrimaryEmail = _.findWhere(senderEmails, { primary: true });
      var accountPrimaryEmailAddress = Meteor.user().primaryEmail;     // 504
      if (_.findWhere(senderEmails, { email: accountPrimaryEmailAddress })) {
        fromEmail = accountPrimaryEmailAddress;                        // 506
      } else if (senderPrimaryEmail) {                                 //
        fromEmail = senderPrimaryEmail.email;                          // 508
      }                                                                //
                                                                       //
      // TODO(soon): In the HTML version, we should display an identity card.
      var identityNote = "";                                           // 512
      if (identity.profile.service === "google") {                     // 513
        identityNote = " (" + identity.privateIntrinsicName + ")";     // 514
      } else if (identity.profile.service === "github") {              //
        identityNote = " (" + identity.profile.intrinsicName + " on GitHub)";
      } else if (identity.profile.service === "email") {               //
        identityNote = " (" + identity.profile.intrinsicName + ")";    // 518
      }                                                                //
                                                                       //
      var message = identity.profile.name + identityNote + " is requesting access to your grain: " + grain.title + ".";
                                                                       //
      var url = origin + "/share/" + grainId + "/" + identityId;       // 524
                                                                       //
      var html = message + "<br><br>" + emailLinkWithInlineStyle(url, "Open Sharing Menu");
                                                                       //
      var user = Meteor.user();                                        // 529
      var ACCESS_REQUEST_LIMIT = 10;                                   // 530
      var resetCount = true;                                           // 531
      if (user.accessRequests) {                                       // 532
        if (user.accessRequests.resetOn < new Date()) {                // 533
          Meteor.users.update({ _id: user._id }, { $unset: { accessRequests: 1 } });
        } else if (user.accessRequests.count >= ACCESS_REQUEST_LIMIT) {
          throw new Meteor.Error(403, "For spam control reasons, you are not allowed to make " + "more than " + ACCESS_REQUEST_LIMIT + " access requests per day.");
        } else {                                                       //
          resetCount = false;                                          // 540
        }                                                              //
      }                                                                //
                                                                       //
      var modifier = { $inc: { "accessRequests.count": 1 } };          // 544
      if (resetCount) {                                                // 545
        var tomorrow = new Date(Date.now() + 86400000);                // 546
        modifier.$set = { "accessRequests.resetOn": tomorrow };        // 547
      }                                                                //
                                                                       //
      Meteor.users.update({ _id: user._id }, modifier);                // 550
                                                                       //
      SandstormEmail.send({                                            // 552
        to: emailAddress,                                              // 553
        envelopeFrom: envelopeFrom,                                    // 554
        from: fromEmail,                                               // 555
        subject: grain.title + " - Request for access",                // 556
        text: message + "\n\nFollow this link to share access:\n\n" + url,
        html: html                                                     // 558
      });                                                              //
    }                                                                  //
  }                                                                    //
});                                                                    //
                                                                       //
if (Meteor.isClient) {                                                 // 564
  (function () {                                                       //
    Tracker.autorun(function () {                                      // 565
      // We need to keep track of certain data about each grain we can view.
      // TODO(cleanup): Do these in GrainView to avoid spurious resubscribes.
      var grains = globalGrains.get();                                 // 568
      grains.forEach(function (grain) {                                // 569
        grain.depend();                                                // 570
        var grainId = grain.grainId();                                 // 571
        if (grainId) {                                                 // 572
          Meteor.subscribe("grainTopBar", grainId);                    // 573
          if (grain.isOwner()) {                                       // 574
            Meteor.subscribe("packageByGrainId", grainId);             // 575
          }                                                            //
                                                                       //
          var token = grain.token();                                   // 578
          if (token) {                                                 // 579
            Meteor.subscribe("tokenInfo", token);                      // 580
          }                                                            //
        }                                                              //
      });                                                              //
    });                                                                //
                                                                       //
    Template.layout.events({                                           // 586
      "click .incognito-button": function (event) {                    // 587
        console.log(event);                                            // 588
        var grains = globalGrains.get();                               // 589
        var token = event.currentTarget.getAttribute("data-token");    // 590
        if (token) {                                                   // 591
          grains.forEach(function (grain) {                            // 592
            if (grain.token() == token) {                              // 593
              grain.doNotRevealIdentity();                             // 594
            }                                                          //
          });                                                          //
        } else {                                                       //
          console.error("Interstitial prompt answered, but no token present?");
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    var promptNewTitle = function () {                                 // 603
      var grain = getActiveGrain(globalGrains.get());                  // 604
      if (grain) {                                                     // 605
        var _prompt = "Set new title:";                                // 606
        if (!grain.isOwner()) {                                        // 607
          _prompt = "Set a new personal title: (does not change the owner's title for this grain)";
        }                                                              //
                                                                       //
        var title = window.prompt(_prompt, grain.title());             // 611
        if (title) {                                                   // 612
          grain.setTitle(title);                                       // 613
        }                                                              //
      }                                                                //
    };                                                                 //
                                                                       //
    Template.grainTitle.events({                                       // 618
      click: function (event) {                                        // 619
        promptNewTitle();                                              // 620
      },                                                               //
                                                                       //
      keydown: function (event) {                                      // 623
        if (event.keyCode === 13 || event.keyCode === 32) {            // 624
          // Allow space or enter to trigger renaming the grain - Firefox doesn't treat enter on the
          // focused element as click().                               //
          promptNewTitle();                                            // 627
          event.preventDefault();                                      // 628
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainDeleteButton.events({                                // 633
      "click button": function (event) {                               // 634
        var grains = globalGrains.get();                               // 635
        var activeIndex = activeGrainIndex(grains);                    // 636
        var activeGrain = grains[activeIndex];                         // 637
        var newActiveIndex = activeIndex == grains.length - 1 ? activeIndex - 1 : activeIndex;
        if (activeGrain.isOwner()) {                                   // 639
          if (window.confirm("Really delete this grain?")) {           // 640
            Meteor.call("deleteGrain", activeGrain.grainId());         // 641
            // TODO: extract globalGrains into a class that has a "close" method for closing the active view
            activeGrain.destroy();                                     // 643
            if (grains.length == 1) {                                  // 644
              globalGrains.set([]);                                    // 645
              Router.go("root"); // will redirect to the right place   // 646
            } else {                                                   //
                grains.splice(activeIndex, 1);                         // 648
                grains[newActiveIndex].setActive(true);                // 649
                globalGrains.set(grains);                              // 650
                Router.go(grains[newActiveIndex].route());             // 651
              }                                                        //
          }                                                            //
        } else {                                                       //
          if (window.confirm("Really forget this grain?")) {           // 655
            var identityId = activeGrain.identityId();                 // 656
            if (identityId) {                                          // 657
              Meteor.call("forgetGrain", activeGrain.grainId(), identityId);
            }                                                          //
            // TODO: extract globalGrains into a class that has a "close" method for closing the active view
            activeGrain.destroy();                                     // 661
            if (grains.length == 1) {                                  // 662
              globalGrains.set([]);                                    // 663
              Router.go("root"); // will redirect to the right place   // 664
            } else {                                                   //
                grains.splice(activeIndex, 1);                         // 666
                grains[newActiveIndex].setActive(true);                // 667
                globalGrains.set(grains);                              // 668
                Router.go(grains[newActiveIndex].route());             // 669
              }                                                        //
          }                                                            //
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainDebugLogButton.events({                              // 676
      "click button": function (event) {                               // 677
        this.reset();                                                  // 678
        var activeGrain = getActiveGrain(globalGrains.get());          // 679
        window.open("/grainlog/" + activeGrain.grainId(), "_blank", "menubar=no,status=no,toolbar=no,width=700,height=700");
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainBackupButton.onCreated(function () {                 // 685
      this.isLoading = new ReactiveVar(false);                         // 686
    });                                                                //
                                                                       //
    Template.grainBackupButton.helpers({                               // 689
      isLoading: function () {                                         // 690
        return Template.instance().isLoading.get();                    // 691
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainBackupButton.events({                                // 695
      "click button": function (event, template) {                     // 696
        template.isLoading.set(true);                                  // 697
        this.reset();                                                  // 698
        var activeGrain = getActiveGrain(globalGrains.get());          // 699
        Meteor.call("backupGrain", activeGrain.grainId(), function (err, id) {
          template.isLoading.set(false);                               // 701
          if (err) {                                                   // 702
            alert("Backup failed: " + err); // TODO(someday): make this better UI
          } else {                                                     //
              // Firefox for some reason decides to kill all websockets when we try to download the file
              // by navigating there. So we're left doing a dirty hack to get around the popup blocker.
              var isFirefox = typeof InstallTrigger !== "undefined";   // 707
                                                                       //
              if (isFirefox) {                                         // 709
                var save = document.createElement("a");                // 710
                save.href = "/downloadBackup/" + id;                   // 711
                                                                       //
                save.download = activeGrain.title() + ".zip";          // 713
                var _event = document.createEvent("MouseEvents");      // 714
                _event.initMouseEvent("click", true, false, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                save.dispatchEvent(_event);                            // 719
              } else {                                                 //
                window.location = "/downloadBackup/" + id;             // 721
              }                                                        //
            }                                                          //
        });                                                            //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainRestartButton.events({                               // 728
      "click button": function (event) {                               // 729
        this.reset();                                                  // 730
        var activeGrain = getActiveGrain(globalGrains.get());          // 731
        var grainId = activeGrain.grainId();                           // 732
                                                                       //
        Meteor.call("shutdownGrain", grainId, function (err) {         // 734
          if (err) {                                                   // 735
            alert("Restart failed: " + err); // TODO(someday): make this better UI
          } else {                                                     //
              var _frames = document.getElementsByClassName("grain-frame");
              for (var i = 0; i < _frames.length; i++) {               // 739
                var frame = _frames[i];                                // 740
                if (frame.dataset.grainid == grainId) {                // 741
                  frame.src = frame.src;                               // 742
                }                                                      //
              }                                                        //
            }                                                          //
        });                                                            //
      }                                                                //
    });                                                                //
                                                                       //
    function selectElementContents(element) {                          // 750
      if (document.body.createTextRange) {                             // 751
        var range = document.body.createTextRange();                   // 752
        range.moveToElementText(element);                              // 753
        range.select();                                                // 754
      } else if (window.getSelection) {                                //
        var selection = window.getSelection();                         // 756
        var range = document.createRange();                            // 757
        range.selectNodeContents(element);                             // 758
        selection.removeAllRanges();                                   // 759
        selection.addRange(range);                                     // 760
      }                                                                //
    }                                                                  //
                                                                       //
    function selectTargetContents(event) {                             // 764
      event.preventDefault();                                          // 765
      selectElementContents(event.currentTarget);                      // 766
    }                                                                  //
                                                                       //
    Template.grainApiTokenPopup.events({                               // 769
      "click .copy-me": selectTargetContents,                          // 770
      "focus .copy-me": selectTargetContents,                          // 771
      "submit .newApiToken": function (event) {                        // 772
        event.preventDefault();                                        // 773
        var activeGrain = getActiveGrain(globalGrains.get());          // 774
        var grainId = activeGrain.grainId();                           // 775
        activeGrain.setGeneratedApiToken("pending");                   // 776
        var roleList = document.getElementById("api-token-role");      // 777
        // TODO(cleanup): avoid using global ids; select a child of the current template instead
        var assignment = { allAccess: null };                          // 779
        if (roleList && roleList.selectedIndex > 0) {                  // 780
          assignment = { roleId: roleList.selectedIndex - 1 };         // 781
        }                                                              //
                                                                       //
        Meteor.call("newApiToken", { identityId: activeGrain.identityId() }, grainId, document.getElementById("api-token-petname").value, assignment, { webkey: { forSharing: false } }, function (error, result) {
          if (error) {                                                 // 788
            activeGrain.setGeneratedApiToken(undefined);               // 789
            window.alert("Failed to create token.\n" + error);         // 790
            console.error(error.stack);                                // 791
          } else {                                                     //
            activeGrain.setGeneratedApiToken(window.location.protocol + "//" + globalDb.makeApiHost(result.token) + "#" + result.token);
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #resetApiToken": function (event) {                       // 800
        var activeGrain = getActiveGrain(globalGrains.get());          // 801
        activeGrain.setGeneratedApiToken(undefined);                   // 802
      },                                                               //
                                                                       //
      "click button.revoke-token": function (event) {                  // 805
        Meteor.call("updateApiToken", event.currentTarget.getAttribute("data-token-id"), { revoked: true });
      },                                                               //
                                                                       //
      "click .token-petname": function (event) {                       // 810
        // TODO(soon): Find a less-annoying way to get this input, perhaps by allowing the user
        //   to edit the petname in place.                             //
        var petname = window.prompt("Set new label:", this.petname);   // 813
        if (petname) {                                                 // 814
          Meteor.call("updateApiToken", event.currentTarget.getAttribute("data-token-id"), { petname: petname });
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainSharePopup.events({                                  // 821
      "click .copy-me": selectTargetContents,                          // 822
      "focus .copy-me": selectTargetContents,                          // 823
      "click #share-grain-popup-closer": function (event) {            // 824
        Session.set("show-share-grain", false);                        // 825
      },                                                               //
                                                                       //
      "click button.who-has-access": function (event, instance) {      // 828
        event.preventDefault();                                        // 829
        var closer = globalTopbar.addItem({                            // 830
          name: "who-has-access",                                      // 831
          template: Template.whoHasAccess,                             // 832
          popupTemplate: Template.whoHasAccessPopup,                   // 833
          data: new ReactiveVar(instance.data),                        // 834
          startOpen: true,                                             // 835
          onDismiss: function () {                                     // 836
            return "remove";                                           // 837
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #privatize-grain": function (event) {                     // 842
        Meteor.call("privatizeGrain", getActiveGrain(globalGrains.get()).grainId());
      },                                                               //
                                                                       //
      "click a.open-non-anonymously": function (event) {               // 846
        event.preventDefault();                                        // 847
        globalTopbar.closePopup();                                     // 848
        getActiveGrain(globalGrains.get()).reset();                    // 849
        getActiveGrain(globalGrains.get()).openSession();              // 850
      }                                                                //
    });                                                                //
                                                                       //
    Template.shareWithOthers.onRendered(function () {                  // 854
      this.find("[role=tab]").focus();                                 // 855
    });                                                                //
                                                                       //
    var activateTargetTab = function (event, instance) {               // 858
      // Deactivate all tabs and all tab panels.                       //
      instance.findAll("ul[role=tablist]>li[role=tab]").forEach(function (element) {
        element.setAttribute("aria-selected", false);                  // 861
      });                                                              //
                                                                       //
      instance.findAll(".tabpanel").forEach(function (element) {       // 864
        element.setAttribute("aria-hidden", true);                     // 865
      });                                                              //
                                                                       //
      // Activate the tab header the user selected.                    //
      event.currentTarget.setAttribute("aria-selected", true);         // 869
      // Show the corresponding tab panel.                             //
      var idToShow = event.currentTarget.getAttribute("aria-controls");
      var tabPanelToShow = instance.find("#" + idToShow);              // 872
      tabPanelToShow.setAttribute("aria-hidden", false);               // 873
    };                                                                 //
                                                                       //
    Template.shareWithOthers.events({                                  // 876
      "click #send-invite-tab-header": activateTargetTab,              // 877
      "click #shareable-link-tab-header": activateTargetTab,           // 878
      "keydown [role=tab]": function (event, template) {               // 879
        if (event.keyCode == 38 || event.keyCode == 40) {              // 880
          // up and down arrows                                        //
          event.preventDefault();                                      // 881
        }                                                              //
                                                                       //
        var $focus = $(template.find(":focus"));                       // 884
        var $items = template.$("[role=tab]:visible");                 // 885
        var focusIndex = $items.index($focus);                         // 886
        var newFocusIndex = undefined;                                 // 887
        if (event.keyCode == 37) {                                     // 888
          // left arrow                                                //
          event.preventDefault();                                      // 889
          newFocusIndex = focusIndex - 1;                              // 890
          if (newFocusIndex == -1) {                                   // 891
            newFocusIndex = $items.length - 1;                         // 892
          }                                                            //
        } else if (event.keyCode == 39) {                              //
          // right arrow                                               //
          event.preventDefault();                                      // 895
          newFocusIndex = focusIndex + 1;                              // 896
          if (newFocusIndex >= $items.length) {                        // 897
            newFocusIndex = 0;                                         // 898
          }                                                            //
        } else if (event.keyCode == 13) {                              //
          // Enter key                                                 //
          event.preventDefault();                                      // 901
          activateTargetTab(event, template);                          // 902
        }                                                              //
                                                                       //
        if (newFocusIndex != null) {                                   // 905
          $items.attr("tabindex", "-1");                               // 906
          var $newFocus = $($items[newFocusIndex]);                    // 907
          $newFocus.attr("tabindex", "0");                             // 908
          $newFocus.focus();                                           // 909
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    Template.shareableLinkTab.events({                                 // 914
      "change .share-token-role": function (event, instance) {         // 915
        var success = instance.completionState.get().success;          // 916
        if (success) {                                                 // 917
          var roleList = event.target;                                 // 918
          var assignment = undefined;                                  // 919
          if (roleList) {                                              // 920
            assignment = { roleId: roleList.selectedIndex };           // 921
          } else {                                                     //
            assignment = { none: null };                               // 923
          }                                                            //
                                                                       //
          Meteor.call("updateApiToken", success.id, { roleAssignment: assignment }, function (error) {
            if (error) {                                               // 927
              console.error(error.stack);                              // 928
            }                                                          //
          });                                                          //
        }                                                              //
      },                                                               //
                                                                       //
      "change .label": function (event, instance) {                    // 934
        var success = instance.completionState.get().success;          // 935
        if (success) {                                                 // 936
          var label = event.target.value;                              // 937
          Meteor.call("updateApiToken", success.id, { petname: label }, function (error) {
            if (error) {                                               // 939
              console.error(error.stack);                              // 940
            }                                                          //
          });                                                          //
        }                                                              //
      },                                                               //
                                                                       //
      "submit form.new-share-token": function (event, instance) {      // 946
        event.preventDefault();                                        // 947
        if (!instance.completionState.get().clear) {                   // 948
          return;                                                      // 949
        }                                                              //
                                                                       //
        var currentGrain = getActiveGrain(globalGrains.get());         // 952
        var grainId = currentGrain.grainId();                          // 953
        var roleList = event.target.getElementsByClassName("share-token-role")[0];
        var assignment = undefined;                                    // 955
        if (roleList) {                                                // 956
          assignment = { roleId: roleList.selectedIndex };             // 957
        } else {                                                       //
          assignment = { none: null };                                 // 959
        }                                                              //
                                                                       //
        instance.completionState.set({ pending: true });               // 962
        Meteor.call("newApiToken", { identityId: currentGrain.identityId() }, grainId, event.target.getElementsByClassName("label")[0].value, assignment, { webkey: { forSharing: true } }, function (error, result) {
          if (error) {                                                 // 967
            console.error(error.stack);                                // 968
          } else {                                                     //
            result.url = getOrigin() + "/shared/" + result.token;      // 970
            instance.completionState.set({ success: result });         // 971
            // On the next render, .copy-me will exist, and we should focus it then.
            Meteor.defer(function () {                                 // 973
              var element = instance.find(".copy-me");                 // 974
              element.focus();                                         // 975
              selectElementContents(element);                          // 976
            });                                                        //
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click .reset-share-token": function (event, instance) {         // 982
        instance.completionState.set({ clear: true });                 // 983
        instance.find("form").reset();                                 // 984
        instance.find("form option[data-default-selected=true]").selected = true;
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainPowerboxOfferPopup.events({                          // 989
      "click button.dismiss": function (event) {                       // 990
        var data = Template.instance().data.get();                     // 991
        data.onDismiss();                                              // 992
      },                                                               //
                                                                       //
      "click .copy-me": selectTargetContents,                          // 995
      "focus .copy-me": selectTargetContents                           // 996
    });                                                                //
                                                                       //
    Template.grainSharePopup.helpers({                                 // 999
      incognito: function () {                                         // 1000
        return !Accounts.getCurrentIdentityId();                       // 1001
      },                                                               //
                                                                       //
      currentTokenUrl: function () {                                   // 1004
        return getOrigin() + "/shared/" + getActiveGrain(globalGrains.get()).token();
      },                                                               //
                                                                       //
      currentGrain: function () {                                      // 1008
        return getActiveGrain(globalGrains.get());                     // 1009
      }                                                                //
    });                                                                //
                                                                       //
    Template.requestAccess.onCreated(function () {                     // 1013
      var _this3 = this;                                               //
                                                                       //
      this._status = new ReactiveVar({ showButton: true });            // 1014
      this._grain = getActiveGrain(globalGrains.get());                // 1015
                                                                       //
      this.autorun(function () {                                       // 1017
        Meteor.userId(); // Read this value so that we resubscribe on login.
        _this3.subscribe("requestingAccess", _this3._grain.grainId());
        var granted = GrantedAccessRequests.findOne({ grainId: _this3._grain.grainId() });
        if (granted) {                                                 // 1021
          _this3._grain.reset(granted.identityId);                     // 1022
          _this3._grain.openSession();                                 // 1023
        }                                                              //
      });                                                              //
    });                                                                //
                                                                       //
    Template.requestAccess.events({                                    // 1028
      "click button.request-access": function (event, instance) {      // 1029
        instance._status.set({ chooseIdentity: true });                // 1030
      }                                                                //
    });                                                                //
                                                                       //
    Template.requestAccess.helpers({                                   // 1034
      status: function () {                                            // 1035
        return Template.instance()._status.get();                      // 1036
      },                                                               //
                                                                       //
      chooseIdentityText: function () {                                // 1039
        if (SandstormDb.getUserIdentityIds(Meteor.user()).length > 1) {
          return "Please select an identity with which to request access.";
        } else {                                                       //
          return "To confirm, please click on your identity below.";   // 1043
        }                                                              //
      },                                                               //
                                                                       //
      identityPickerData: function () {                                // 1047
        var identities = SandstormDb.getUserIdentityIds(Meteor.user()).map(function (id) {
          return globalDb.getIdentity(id);                             //
        });                                                            //
        var instance = Template.instance();                            // 1050
        function onPicked(identityId) {                                // 1051
          var grainId = instance._grain.grainId();                     // 1052
          Meteor.call("requestAccess", getOrigin(), grainId, identityId, function (error, result) {
            if (error) {                                               // 1054
              instance._status.set({ error: error });                  // 1055
            } else {                                                   //
              instance._status.set({ success: true });                 // 1057
            }                                                          //
          });                                                          //
                                                                       //
          instance._status.set({ waiting: true });                     // 1061
        }                                                              //
                                                                       //
        return {                                                       // 1064
          identities: identities,                                      // 1065
          onPicked: onPicked                                           // 1066
        };                                                             //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainView.helpers({                                       // 1071
      unpackedGrainState: function () {                                // 1072
        return mapGrainStateToTemplateData(this);                      // 1073
      },                                                               //
                                                                       //
      identityPickerData: function () {                                // 1076
        var identities = SandstormDb.getUserIdentityIds(Meteor.user()).map(function (id) {
          var identity = Meteor.users.findOne({ _id: id });            // 1078
          if (identity) {                                              // 1079
            SandstormDb.fillInProfileDefaults(identity);               // 1080
            SandstormDb.fillInIntrinsicName(identity);                 // 1081
            SandstormDb.fillInPictureUrl(identity);                    // 1082
            return identity;                                           // 1083
          }                                                            //
        });                                                            //
                                                                       //
        var grain = getActiveGrain(globalGrains.get());                // 1087
        return { identities: identities,                               // 1088
          onPicked: function (identityId) {                            // 1089
            grain.revealIdentity(identityId);                          // 1089
          } };                                                         //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grain.helpers({                                           // 1093
      currentGrain: function () {                                      // 1094
        return getActiveGrain(globalGrains.get());                     // 1095
      },                                                               //
                                                                       //
      isOwner: function () {                                           // 1098
        var current = getActiveGrain(globalGrains.get());              // 1099
        return current && current.isOwner();                           // 1100
      },                                                               //
                                                                       //
      grainSize: function () {                                         // 1103
        var current = getActiveGrain(globalGrains.get());              // 1104
        var sizeBytes = current && current.size();                     // 1105
        if (sizeBytes !== undefined) {                                 // 1106
          return "(" + prettySize(sizeBytes) + ")";                    // 1107
        } else {                                                       //
          return "";                                                   // 1109
        }                                                              //
      },                                                               //
                                                                       //
      setGrainWindowTitle: function () {                               // 1113
        var current = getActiveGrain(globalGrains.get());              // 1114
        if (current) {                                                 // 1115
          current.updateDocumentTitle();                               // 1116
        }                                                              //
      },                                                               //
                                                                       //
      displayWebkeyButton: function () {                               // 1120
        // TODO: figure out what this should do                        //
        return Meteor.userId() || !this.oldSharingModel;               // 1122
      },                                                               //
                                                                       //
      showPowerboxOffer: function () {                                 // 1125
        var current = getActiveGrain(globalGrains.get());              // 1126
        return current && current.showPowerboxOffer();                 // 1127
      },                                                               //
                                                                       //
      powerboxOfferData: function () {                                 // 1130
        var current = getActiveGrain(globalGrains.get());              // 1131
        return current && current.powerboxOfferData();                 // 1132
      },                                                               //
                                                                       //
      showPowerboxRequest: function () {                               // 1135
        var current = getActiveGrain(globalGrains.get());              // 1136
        return current && current.showPowerboxRequest();               // 1137
      },                                                               //
                                                                       //
      powerboxRequestData: function () {                               // 1140
        var current = getActiveGrain(globalGrains.get());              // 1141
        return current && current.powerboxRequestData();               // 1142
      },                                                               //
                                                                       //
      cancelPowerboxRequest: function () {                             // 1145
        return function () {                                           // 1146
          var current = getActiveGrain(globalGrains.get());            // 1147
          current.setPowerboxRequest(undefined);                       // 1148
          return "remove";                                             // 1149
        };                                                             //
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainTitle.helpers({                                      // 1154
      title: function () {                                             // 1155
        var grain = getActiveGrain(globalGrains.get());                // 1156
        return grain && grain.title() || "Untitled grain";             // 1157
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainApiTokenPopup.helpers({                              // 1161
      displayToken: function () {                                      // 1162
        return !this.revoked && !this.expiresIfUnused && !this.parentToken;
      },                                                               //
                                                                       //
      existingTokens: function () {                                    // 1166
        var current = getActiveGrain(globalGrains.get());              // 1167
        return current && ApiTokens.find({                             // 1168
          grainId: current.grainId(),                                  // 1169
          identityId: current.identityId(),                            // 1170
          forSharing: { $ne: true },                                   // 1171
          $or: [{ owner: { webkey: null } }, { owner: { $exists: false } }],
          expiresIfUnused: null                                        // 1176
        });                                                            //
      },                                                               //
                                                                       //
      generatedApiToken: function () {                                 // 1180
        var current = getActiveGrain(globalGrains.get());              // 1181
        return current && current.generatedApiToken();                 // 1182
      },                                                               //
                                                                       //
      generatedApiTokenPending: function () {                          // 1185
        var current = getActiveGrain(globalGrains.get());              // 1186
        return (current && current.generatedApiToken()) == "pending";  // 1187
      },                                                               //
                                                                       //
      viewInfo: function () {                                          // 1190
        var current = getActiveGrain(globalGrains.get());              // 1191
        return current && current.viewInfo();                          // 1192
      }                                                                //
    });                                                                //
                                                                       //
    Template.whoHasAccessPopup.onCreated(function () {                 // 1196
      var _this = this;                                                // 1197
      this.subscribe("contactProfiles");                               // 1198
      var currentGrain = getActiveGrain(globalGrains.get());           // 1199
      _this.identityId = currentGrain.identityId();                    // 1200
      _this.grainId = currentGrain.grainId();                          // 1201
      _this.transitiveShares = new ReactiveVar(null);                  // 1202
      _this.downstreamTokensById = new ReactiveVar({});                // 1203
      this.resetTransitiveShares = function () {                       // 1204
        Meteor.call("transitiveShares", _this.identityId, _this.grainId, function (error, downstream) {
          if (error) {                                                 // 1207
            console.error(error.stack);                                // 1208
          } else {                                                     //
            (function () {                                             //
              var downstreamTokensById = {};                           // 1210
              var sharesByRecipient = {};                              // 1211
              downstream.forEach(function (token) {                    // 1212
                downstreamTokensById[token._id] = token;               // 1213
                if (Match.test(token.owner, { user: Match.ObjectIncluding({ identityId: String }) })) {
                  var recipient = token.owner.user.identityId;         // 1215
                  if (!sharesByRecipient[recipient]) {                 // 1216
                    sharesByRecipient[recipient] = {                   // 1217
                      recipient: recipient,                            // 1218
                      dedupedShares: [],                               // 1219
                      allShares: []                                    // 1220
                    };                                                 //
                  }                                                    //
                                                                       //
                  sharesByRecipient[recipient].allShares.push(token);  // 1224
                  var dedupedShares = sharesByRecipient[recipient].dedupedShares;
                  if (!dedupedShares.some(function (share) {           // 1226
                    return share.identityId === token.identityId;      //
                  })) {                                                //
                    dedupedShares.push(token);                         // 1227
                  }                                                    //
                }                                                      //
              });                                                      //
                                                                       //
              var result = _.values(sharesByRecipient);                // 1232
              if (result.length == 0) {                                // 1233
                result = { empty: true };                              // 1234
              }                                                        //
                                                                       //
              _this.transitiveShares.set(result);                      // 1237
              _this.downstreamTokensById.set(downstreamTokensById);    // 1238
            })();                                                      //
          }                                                            //
        });                                                            //
      };                                                               //
                                                                       //
      this.resetTransitiveShares();                                    // 1243
    });                                                                //
                                                                       //
    Template.whoHasAccessPopup.events({                                // 1246
      "change .share-token-role": function (event, instance) {         // 1247
        var roleList = event.target;                                   // 1248
        var assignment = undefined;                                    // 1249
        if (roleList) {                                                // 1250
          assignment = { roleId: roleList.selectedIndex };             // 1251
        } else {                                                       //
          assignment = { none: null };                                 // 1253
        }                                                              //
                                                                       //
        Meteor.call("updateApiToken", roleList.getAttribute("data-token-id"), { roleAssignment: assignment }, function (error) {
          if (error) {                                                 // 1258
            console.error(error.stack);                                // 1259
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click button.revoke-token": function (event, instance) {        // 1264
        Meteor.call("updateApiToken", event.currentTarget.getAttribute("data-token-id"), { revoked: true });
        instance.resetTransitiveShares();                              // 1267
      },                                                               //
                                                                       //
      "click button.revoke-access": function (event, instance) {       // 1270
        var recipient = event.currentTarget.getAttribute("data-recipient");
        var transitiveShares = instance.transitiveShares.get();        // 1272
        var tokensById = instance.downstreamTokensById.get();          // 1273
        var recipientShares = _.findWhere(transitiveShares, { recipient: recipient });
        var currentIdentityId = Accounts.getCurrentIdentityId();       // 1275
        var recipientTokens = _.where(recipientShares.allShares, { identityId: currentIdentityId });
                                                                       //
        // Two cases:                                                  //
        // 1. All of the tokens are direct shares. Easy. Just revoke them.
        // 2. Some of the links are child tokens. For each, we walk up the chain of parents to the
        //    root. Then we walk downwards to see whether any other identities would be immediately
        //    affected by revoking those roots. We then collect that data and display it in a
        //    confirmation dialog.                                     //
                                                                       //
        var directTokens = [];                                         // 1285
        // Direct shares from the current identity to the recipient.   //
                                                                       //
        var rootTokens = [];                                           // 1288
        // Roots of chains of child tokens starting at the current identity and leading to the
        // recipient.                                                  //
                                                                       //
        recipientTokens.forEach(function (token) {                     // 1292
          if (token.parentToken) {                                     // 1293
            var currentToken = token;                                  // 1294
            do {                                                       // 1295
              currentToken = tokensById[token.parentToken];            // 1296
            } while (currentToken.parentToken);                        //
                                                                       //
            rootTokens.push(currentToken);                             // 1299
          } else {                                                     //
            directTokens.push(token);                                  // 1301
          }                                                            //
        });                                                            //
                                                                       //
        if (rootTokens.length > 0) {                                   // 1305
          var _ret5 = (function () {                                   //
            // Some of the shares are not direct.                      //
                                                                       //
            var tokensByParent = {};                                   // 1308
            for (var id in babelHelpers.sanitizeForInObject(tokensById)) {
              var token = tokensById[id];                              // 1310
              if (token.parentToken) {                                 // 1311
                if (!tokensByParent[token.parentToken]) {              // 1312
                  tokensByParent[token.parentToken] = [];              // 1313
                }                                                      //
                                                                       //
                tokensByParent[token.parentToken].push(token);         // 1316
              }                                                        //
            }                                                          //
                                                                       //
            var otherAffectedIdentities = {};                          // 1320
            var tokenStack = rootTokens.slice(0);                      // 1321
            while (tokenStack.length > 0) {                            // 1322
              var current = tokenStack.pop();                          // 1323
              if (Match.test(current.owner, { user: Match.ObjectIncluding({ identityId: String }) })) {
                if (current.owner.user.identityId != recipient) {      // 1326
                  otherAffectedIdentities[current.owner.user.identityId] = true;
                }                                                      //
              } else {                                                 //
                if (tokensByParent[current._id]) {                     // 1330
                  var children = tokensByParent[current._id];          // 1331
                  children.forEach(function (child) {                  // 1332
                    tokenStack.push(child);                            // 1333
                  });                                                  //
                }                                                      //
              }                                                        //
            }                                                          //
                                                                       //
            var recipientIdentity = ContactProfiles.findOne({ _id: recipient });
            var recipientName = recipientIdentity && recipientIdentity.profile.name || "Unknown User";
            var singular = rootTokens.length == 1;                     // 1342
                                                                       //
            var tokenLabels = _.pluck(rootTokens, "petname").map(function (petname) {
              return petname || "Unlabeled Link";                      //
            }).map(function (petname) {                                //
              return "\"" + petname + "\"";                            //
            }).join(", ");                                             //
                                                                       //
            var confirmText = "This will revoke the following sharing link" + (singular ? "" : "s") + ":\n\n    " + tokenLabels + "\n\n";
                                                                       //
            var othersNote = "(No signed-in user other than " + recipientName + " has opened " + (singular ? "this link" : "these links") + " yet.)";
            if (Object.keys(otherAffectedIdentities).length > 0) {     // 1354
              var othersNames = Object.keys(otherAffectedIdentities).map(function (identityId) {
                return ContactProfiles.findOne({ _id: identityId });   //
              }).map(function (identity) {                             //
                return identity && identity.profile.name || "Unknown User";
              }).join(", ");                                           //
              othersNote = (singular ? "This link has" : "These links have") + " also been opened by:\n\n    " + othersNames;
            }                                                          //
                                                                       //
            if (window.confirm(confirmText + othersNote)) {            // 1363
              rootTokens.forEach(function (token) {                    // 1364
                Meteor.call("updateApiToken", token._id, { revoked: true });
              });                                                      //
            } else {                                                   //
              // Cancel.                                               //
              return {                                                 // 1369
                v: undefined                                           //
              };                                                       //
            }                                                          //
          })();                                                        //
                                                                       //
          if (typeof _ret5 === "object") return _ret5.v;               //
        }                                                              //
                                                                       //
        directTokens.forEach(function (token) {                        // 1373
          Meteor.call("updateApiToken", token._id, { revoked: true });
        });                                                            //
                                                                       //
        instance.resetTransitiveShares();                              // 1377
      },                                                               //
                                                                       //
      "click .token-petname": function (event, instance) {             // 1380
        // TODO(soon): Find a less-annoying way to get this input, perhaps by allowing the user
        //   to edit the petname in place.                             //
        var petname = window.prompt("Set new label:", this.petname);   // 1383
        if (petname) {                                                 // 1384
          Meteor.call("updateApiToken", event.currentTarget.getAttribute("data-token-id"), { petname: petname });
        }                                                              //
                                                                       //
        instance.resetTransitiveShares();                              // 1389
      }                                                                //
    });                                                                //
                                                                       //
    function isEmptyPermissionSet(permissionSet) {                     // 1393
      if (!permissionSet) {                                            // 1394
        return true;                                                   // 1395
      }                                                                //
                                                                       //
      for (var ii = 0; ii < permissionSet.length; ++ii) {              // 1398
        if (permissionSet[ii]) {                                       // 1399
          return false;                                                // 1400
        }                                                              //
      }                                                                //
                                                                       //
      return true;                                                     // 1404
    }                                                                  //
                                                                       //
    Template.whoHasAccessPopup.helpers({                               // 1407
      existingShareTokens: function () {                               // 1408
        if (Meteor.userId()) {                                         // 1409
          return ApiTokens.find({                                      // 1410
            grainId: Template.instance().grainId,                      // 1411
            identityId: Template.instance().identityId,                // 1412
            forSharing: true,                                          // 1413
            $or: [{ owner: { webkey: null } }, { owner: { $exists: false } }]
          }).fetch();                                                  //
        }                                                              //
      },                                                               //
                                                                       //
      isCurrentIdentity: function () {                                 // 1422
        if (this.identityId === Accounts.getCurrentIdentityId()) {     // 1423
          return true;                                                 // 1424
        }                                                              //
      },                                                               //
                                                                       //
      getPetname: function () {                                        // 1428
        if (this.petname) {                                            // 1429
          return this.petname;                                         // 1430
        } else {                                                       //
          return "Unlabeled Link";                                     // 1432
        }                                                              //
      },                                                               //
                                                                       //
      displayName: function (identityId) {                             // 1436
        var identity = ContactProfiles.findOne({ _id: identityId });   // 1437
        if (!identity) {                                               // 1438
          identity = Meteor.users.findOne({ _id: identityId });        // 1439
        }                                                              //
                                                                       //
        if (identity) {                                                // 1442
          return identity.profile.name;                                // 1443
        } else {                                                       //
          return "Unknown User (" + identityId.slice(0, 16) + ")";     // 1445
        }                                                              //
      },                                                               //
                                                                       //
      transitiveShares: function () {                                  // 1449
        return Template.instance().transitiveShares.get();             // 1450
      },                                                               //
                                                                       //
      indexedRoles: function () {                                      // 1453
        var result = [];                                               // 1454
        var instance = Template.instance();                            // 1455
        var currentGrain = getActiveGrain(globalGrains.get());         // 1456
        var roles = currentGrain.viewInfo().roles;                     // 1457
        for (var ii = 0; ii < roles.length; ++ii) {                    // 1458
          result.push({ idx: ii, title: roles[ii].title, verbPhrase: roles[ii].verbPhrase });
        }                                                              //
                                                                       //
        return result;                                                 // 1462
      },                                                               //
                                                                       //
      roleText: function () {                                          // 1465
        if (this.verbPhrase) {                                         // 1466
          return this.verbPhrase.defaultText;                          // 1467
        } else {                                                       //
          return "is " + this.title.defaultText;                       // 1469
        }                                                              //
      },                                                               //
                                                                       //
      hasCustomRole: function (token) {                                // 1473
        var role = token.roleAssignment;                               // 1474
        if ("roleId" in role && isEmptyPermissionSet(role.addPermissions) && isEmptyPermissionSet(role.removePermissions)) {
          return false;                                                // 1478
        }                                                              //
                                                                       //
        return true;                                                   // 1481
      },                                                               //
                                                                       //
      hasCurrentRole: function (token) {                               // 1484
        var role = token.roleAssignment;                               // 1485
        if ("roleId" in role && role.roleId == this.idx && isEmptyPermissionSet(role.addPermissions) && isEmptyPermissionSet(role.removePermissions)) {
          return true;                                                 // 1489
        }                                                              //
                                                                       //
        return false;                                                  // 1492
      },                                                               //
                                                                       //
      displayToken: function () {                                      // 1495
        return !this.revoked && !this.expiresIfUnused && !this.parentToken;
      },                                                               //
                                                                       //
      viewInfo: function () {                                          // 1499
        var activeGrain = getActiveGrain(globalGrains.get());          // 1500
        return activeGrain && activeGrain.viewInfo();                  // 1501
      }                                                                //
    });                                                                //
                                                                       //
    Template.selectRole.helpers({                                      // 1505
      roleText: function () {                                          // 1506
        if (this.verbPhrase) {                                         // 1507
          return this.verbPhrase.defaultText;                          // 1508
        } else {                                                       //
          return "is " + this.title.defaultText;                       // 1510
        }                                                              //
      }                                                                //
    });                                                                //
                                                                       //
    Template.shareableLinkTab.onCreated(function () {                  // 1515
      this.completionState = new ReactiveVar({ clear: true });         // 1516
    });                                                                //
                                                                       //
    Template.emailInviteTab.onCreated(function () {                    // 1519
      this.completionState = new ReactiveVar({ clear: true });         // 1520
      this.contacts = new ReactiveVar([]);                             // 1521
      this.grain = getActiveGrain(globalGrains.get());                 // 1522
    });                                                                //
                                                                       //
    Template.emailInviteTab.onDestroyed(function () {                  // 1525
      Session.set("share-grain-" + Template.instance().grain.grainId(), undefined);
    });                                                                //
                                                                       //
    Template.shareableLinkTab.helpers({                                // 1529
      completionState: function () {                                   // 1530
        var instance = Template.instance();                            // 1531
        return instance.completionState.get();                         // 1532
      }                                                                //
    });                                                                //
    Template.emailInviteTab.helpers({                                  // 1535
      completionState: function () {                                   // 1536
        var instance = Template.instance();                            // 1537
        return instance.completionState.get();                         // 1538
      },                                                               //
                                                                       //
      contacts: function () {                                          // 1541
        return Template.instance().contacts;                           // 1542
      },                                                               //
                                                                       //
      preselectedIdentityId: function () {                             // 1545
        return Session.get("share-grain-" + Template.instance().grain.grainId());
      }                                                                //
    });                                                                //
                                                                       //
    Template.emailInviteTab.events({                                   // 1550
      "submit form.email-invite": function (event, instance) {         // 1551
        event.preventDefault();                                        // 1552
        return false;                                                  // 1553
      },                                                               //
                                                                       //
      "click form.email-invite button": function (event, instance) {   // 1556
        event.preventDefault();                                        // 1557
        if (!instance.completionState.get().clear) {                   // 1558
          return;                                                      // 1559
        }                                                              //
                                                                       //
        var grainId = instance.data.grainId;                           // 1562
        var title = instance.data.title;                               // 1563
                                                                       //
        var roleList = instance.find(".share-token-role");             // 1565
        var assignment = undefined;                                    // 1566
        if (roleList) {                                                // 1567
          assignment = { roleId: roleList.selectedIndex };             // 1568
        } else {                                                       //
          assignment = { none: null };                                 // 1570
        }                                                              //
                                                                       //
        var message = instance.find(".personal-message").value;        // 1573
        instance.completionState.set({ pending: true });               // 1574
                                                                       //
        var currentGrain = getActiveGrain(globalGrains.get());         // 1576
                                                                       //
        // HTML-escape the message.                                    //
        var div = document.createElement("div");                       // 1579
        div.appendChild(document.createTextNode(message));             // 1580
        var htmlMessage = div.innerHTML.replace(/\n/g, "<br>");        // 1581
                                                                       //
        var contacts = instance.contacts.get();                        // 1583
        var emails = instance.find("input.emails");                    // 1584
        var emailsValue = emails.value;                                // 1585
        if (emailsValue) {                                             // 1586
          if (emailsValue.match(/.+\@.+\..+/)) {                       // 1587
            contacts.push({                                            // 1588
              _id: emailsValue,                                        // 1589
              profile: {                                               // 1590
                service: "email",                                      // 1591
                name: emailsValue,                                     // 1592
                intrinsicName: emailsValue,                            // 1593
                pictureUrl: "/email.svg"                               // 1594
              },                                                       //
              isDefault: true                                          // 1596
            });                                                        //
            instance.contacts.set(contacts);                           // 1598
            emails.value = "";                                         // 1599
          } else {                                                     //
            instance.completionState.set({ error: "Invalid text in contact input box: " + emailsValue });
            return;                                                    // 1603
          }                                                            //
        }                                                              //
                                                                       //
        Meteor.call("inviteUsersToGrain", getOrigin(), currentGrain.identityId(), grainId, title, assignment, contacts, { text: message, html: htmlMessage }, function (error, result) {
          if (error) {                                                 // 1610
            instance.completionState.set({ error: error.toString() });
          } else {                                                     //
            if (result.failures.length > 0) {                          // 1613
              var _message = "Failed to send to: ";                    // 1614
              for (var ii = 0; ii < result.failures.length; ++ii) {    // 1615
                console.error(result.failures[ii].error);              // 1616
                if (ii != 0) {                                         // 1617
                  _message += ", ";                                    // 1618
                }                                                      //
                                                                       //
                _message += result.failures[ii].contact.profile.name;  // 1621
                var warning = result.failures[ii].warning;             // 1622
                if (warning) {                                         // 1623
                  _message += ". " + warning;                          // 1624
                }                                                      //
              }                                                        //
                                                                       //
              instance.completionState.set({ error: _message });       // 1628
            } else {                                                   //
              instance.completionState.set({ success: "success" });    // 1630
            }                                                          //
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click .reset-invite": function (event, instance) {              // 1636
        instance.contacts.set([]);                                     // 1637
        instance.completionState.set({ clear: true });                 // 1638
        instance.find("form").reset();                                 // 1639
        instance.find("form option[data-default-selected=true]").selected = true;
      },                                                               //
                                                                       //
      "click .start-over-invite": function (event, instance) {         // 1643
        instance.completionState.set({ clear: true });                 // 1644
      }                                                                //
    });                                                                //
                                                                       //
    Template.grainPowerboxOfferPopup.helpers({                         // 1648
      powerboxOfferUrl: function () {                                  // 1649
        var offer = Template.instance().data.get().offer;              // 1650
        return offer && offer.url;                                     // 1651
      }                                                                //
    });                                                                //
                                                                       //
    // Send a keep-alive for each grain every now and then.            //
    Meteor.setInterval(function () {                                   // 1656
      var grains = globalGrains.get();                                 // 1657
      if (!grains) return;                                             // 1658
                                                                       //
      // Meteor has an exponential backoff of up to 5 minutes for reconnect. This is unnacceptable
      // for us, since we rely on Sessions being re-established in under 60s.
      if (Meteor.status().status === "waiting") {                      // 1662
        console.log("Sandstorm is trying to reconnect...");            // 1663
        Meteor.reconnect();                                            // 1664
      }                                                                //
                                                                       //
      grains.forEach(function (grain) {                                // 1667
        if (grain.sessionId()) {                                       // 1668
          // TODO(soon):  Investigate what happens in background tabs.  Maybe arrange to re-open the
          //   app if it dies while in the background.                 //
          Meteor.call("keepSessionAlive", grain.sessionId(), function (error, result) {
            // Sessions will automatically resume if possible, otherwise they will refresh.
          });                                                          //
        }                                                              //
      });                                                              //
    }, 60000);                                                         //
                                                                       //
    var memoizedNewApiToken = {};                                      // 1678
    // Maps sha256(JSON.stringify(parameters)) -> {timestamp, promise}
    //                                                                 //
    // This memoizes calls to the Meteor method "newApiToken", so that multiple calls in rapid
    // succession will not create multiple tokens. `parameters` above is an array containing the
    // calls parameters in order (excluding the method name and callback), and `promise` is a promise
    // for the result of the call.                                     //
                                                                       //
    // Message handler for Sandstorm's client-side postMessage API.    //
    Meteor.startup(function () {                                       // 1687
      var messageListener = function (event) {                         // 1688
        if (event.origin === getOrigin()) {                            // 1689
          // Meteor likes to postMessage() to itself sometimes, so we ignore these messages.
          return;                                                      // 1691
        }                                                              //
                                                                       //
        // Look up the grain index of which grain this postmessage came from, so we can map behavior
        // into that particular grain's state                          //
        var grains = globalGrains.get();                               // 1696
        var senderGrainIndex = grainOriginToIndex(grains, event.origin);
        if (senderGrainIndex == -1) {                                  // 1698
          // We got a postMessage from an origin that is not a grain we currently believe is open.
          // Ignore it. (It may be aimed at some other message listener registered elsewhere...)
          return;                                                      // 1701
        }                                                              //
                                                                       //
        var senderGrain = grains[senderGrainIndex];                    // 1704
                                                                       //
        if (event.data.setPath || event.data.setPath === "") {         // 1706
          var path = event.data.setPath || "/";                        // 1707
          check(path, String);                                         // 1708
          check(path.charAt(0), "/");                                  // 1709
          // TODO(security): More sanitization of this path. E.g. reject "/../../".
          senderGrain.setPath(path);                                   // 1711
        } else if (event.data.startSharing) {                          //
          // Allow the current grain to request that the "Share Access" menu be shown.
          // Only show this popup if no other popup is currently active.
          // TODO(security): defend against malicious apps spamming this call, blocking all other UI.
          var currentGrain = getActiveGrain(globalGrains.get());       // 1716
          if (senderGrain === currentGrain && !globalTopbar.isPopupOpen()) {
            globalTopbar.openPopup("share");                           // 1718
          }                                                            //
        } else if (event.data.setTitle || event.data.setTitle === "") {
          senderGrain.setFrameTitle(event.data.setTitle);              // 1721
        } else if (event.data.renderTemplate) {                        //
          var _ret6 = (function () {                                   //
            // Request creation of a single-use template with a privileged API token.
            // Why?  Apps should not be able to obtain capabilities-as-keys to
            // themselves directly, because those can be leaked through an
            // arbitrary bit stream or covert channel.  However, apps often need a
            // way to provide instructions to users to copy/paste with some
            // privileged token contained within.  By providing this templating in
            // the platform, we can ensure that the token is only visible to the
            // shell's origin.                                         //
            var call = event.data.renderTemplate;                      // 1731
            check(call, Object);                                       // 1732
            var rpcId = call.rpcId;                                    // 1733
            try {                                                      // 1734
              check(call, {                                            // 1735
                rpcId: String,                                         // 1736
                template: String,                                      // 1737
                petname: Match.Optional(String),                       // 1738
                roleAssignment: Match.Optional(roleAssignmentPattern),
                forSharing: Match.Optional(Boolean),                   // 1740
                clipboardButton: Match.Optional(Match.OneOf(undefined, null, "left", "right")),
                unauthenticated: Match.Optional(Object)                // 1742
              });                                                      //
            } // Note: `unauthenticated` will be validated on the server. We just pass it through
            //   here.                                                 //
            catch (error) {                                            // 1746
              event.source.postMessage({ rpcId: rpcId, error: error.toString() }, event.origin);
              return {                                                 // 1748
                v: undefined                                           //
              };                                                       //
            }                                                          //
                                                                       //
            var template = call.template;                              // 1751
            var petname = "connected external app";                    // 1752
            if (call.petname) {                                        // 1753
              petname = call.petname;                                  // 1754
            }                                                          //
                                                                       //
            var assignment = { allAccess: null };                      // 1757
            var clipboardButton = call.clipboardButton;                // 1758
            if (call.roleAssignment) {                                 // 1759
              assignment = call.roleAssignment;                        // 1760
            }                                                          //
                                                                       //
            var forSharing = call.forSharing ? call.forSharing : false;
            // Tokens expire by default in 5 minutes from generation date
            var selfDestructDuration = 5 * 60 * 1000;                  // 1765
                                                                       //
            var provider = undefined;                                  // 1767
            if (Router.current().route.getName() === "shared") {       // 1768
              provider = { rawParentToken: Router.current().params.token };
            } else {                                                   //
              provider = { identityId: senderGrain.identityId() };     // 1771
            }                                                          //
                                                                       //
            var owner = {                                              // 1774
              webkey: {                                                // 1775
                forSharing: forSharing,                                // 1776
                expiresIfUnusedDuration: selfDestructDuration          // 1777
              }                                                        //
            };                                                         //
                                                                       //
            var params = [provider, senderGrain.grainId(), petname, assignment, owner, call.unauthenticated];
                                                                       //
            var memoizeKey = SHA256(JSON.stringify(params));           // 1785
            var memoizeResult = memoizedNewApiToken[memoizeKey];       // 1786
            if (memoizeResult && Date.now() - memoizeResult.timestamp > selfDestructDuration / 2) {
              // Memoized result is too old. Discard.                  //
              memoizeResult = undefined;                               // 1789
            }                                                          //
                                                                       //
            if (!memoizeResult) {                                      // 1792
              memoizedNewApiToken[memoizeKey] = memoizeResult = {      // 1793
                timestamp: Date.now(),                                 // 1794
                promise: new Promise(function (resolve, reject) {      // 1795
                  var callback = function (err, result) {              // 1796
                    if (err) {                                         // 1797
                      reject(err);                                     // 1798
                    } else {                                           //
                      resolve(result);                                 // 1800
                    }                                                  //
                  };                                                   //
                                                                       //
                  Meteor.call.apply(Meteor, ["newApiToken"].concat(params, callback));
                })                                                     //
              };                                                       //
            }                                                          //
                                                                       //
            memoizeResult.promise.then(function (result) {             // 1809
              var tokenId = result.token;                              // 1810
              // Generate random key id2.                              //
              var id2 = Random.secret();                               // 1812
              // Store apitoken id1 and template in session storage in the offer
              // template namespace under key id2.                     //
              var key = "offerTemplate" + id2;                         // 1815
              var host = globalDb.makeApiHost(tokenId);                // 1816
              var renderedTemplate = template.replace(/\$API_TOKEN/g, tokenId).replace(/\$API_HOST/g, host);
              sessionStorage.setItem(key, JSON.stringify({             // 1819
                token: tokenId,                                        // 1820
                renderedTemplate: renderedTemplate,                    // 1821
                clipboardButton: clipboardButton,                      // 1822
                expires: Date.now() + selfDestructDuration,            // 1823
                host: host                                             // 1824
              }));                                                     //
                                                                       //
              // Send message to event.source with URL containing id2  //
              templateLink = window.location.origin + "/offer-template.html#" + id2;
              event.source.postMessage({ rpcId: rpcId, uri: templateLink }, event.origin);
            }, function (error) {                                      //
              event.source.postMessage({ rpcId: rpcId, error: error.toString() }, event.origin);
            });                                                        //
          })();                                                        //
                                                                       //
          if (typeof _ret6 === "object") return _ret6.v;               //
        } else if (event.data.powerboxRequest) {                       //
          var powerboxRequest = event.data.powerboxRequest;            // 1835
          check(powerboxRequest, {                                     // 1836
            // TODO: be more strict, and check more fields, once the test apps are more conformant
            rpcId: Match.Any,                                          // 1838
            query: Match.Optional([Object]), // TODO: be more precise, and also make non-optional once tests are updated to provide a query
            saveLabel: Match.Optional(String)                          // 1840
          });                                                          //
                                                                       //
          var powerboxRequestInfo = {                                  // 1843
            source: event.source,                                      // 1844
            origin: event.origin,                                      // 1845
            // We'll need these to post a reply message                //
                                                                       //
            rpcId: powerboxRequest.rpcId,                              // 1848
            query: powerboxRequest.query,                              // 1849
            saveLabel: powerboxRequest.saveLabel,                      // 1850
            // These data come from the grain                          //
                                                                       //
            sessionId: senderGrain.sessionId(),                        // 1853
            grainId: senderGrain.grainId(),                            // 1854
            identityId: senderGrain.identityId(),                      // 1855
            // Attach grain context to the request.                    //
                                                                       //
            onCompleted: function () {                                 // 1858
              globalTopbar.closePopup();                               // 1858
            }                                                          //
          };                                                           //
          // Allow the grain to close the popup when we've completed the request.
          var requestContext = new SandstormPowerboxRequest(globalDb, powerboxRequestInfo);
          senderGrain.setPowerboxRequest(requestContext);              // 1862
          globalTopbar.openPopup("request");                           // 1863
        } else {                                                       //
          console.log("postMessage from app not understood: ", event.data);
          console.log(event);                                          // 1866
        }                                                              //
      };                                                               //
                                                                       //
      window.addEventListener("message", messageListener, false);      // 1870
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
if (Meteor.isClient) {                                                 // 1874
  function maybeScrollLog() {                                          // 1875
    var elem = document.getElementById("grainLog");                    // 1876
    if (elem) {                                                        // 1877
      // The log already exists. It's about to be updated. Check if it's scrolled to the bottom
      // before the update.                                            //
      if (elem.scrollHeight - elem.scrollTop === elem.clientHeight) {  // 1880
        // Indeed, so we want to scroll it back to the bottom after the update.
        Tracker.afterFlush(function () {                               // 1882
          scrollLogToBottom(elem);                                     // 1882
        });                                                            //
      }                                                                //
    } else {                                                           //
      // No element exists yet, but it's probably about to be created, in which case we definitely
      // want to scroll it.                                            //
      Tracker.afterFlush(function () {                                 // 1887
        var elem2 = document.getElementById("grainLog");               // 1888
        if (elem2) scrollLogToBottom(elem2);                           // 1889
      });                                                              //
    }                                                                  //
  }                                                                    //
                                                                       //
  function scrollLogToBottom(elem) {                                   // 1894
    elem.scrollTop = elem.scrollHeight;                                // 1895
  }                                                                    //
}                                                                      //
                                                                       //
function makeGrainIdActive(grainId) {                                  // 1899
  var grains = globalGrains.get();                                     // 1900
  for (var i = 0; i < grains.length; i++) {                            // 1901
    var grain = grains[i];                                             // 1902
    if (grain.grainId() === grainId) {                                 // 1903
      if (!grain.isActive()) {                                         // 1904
        grain.setActive(true);                                         // 1905
      }                                                                //
    } else {                                                           //
      if (grain.isActive()) {                                          // 1908
        grain.setActive(false);                                        // 1909
      }                                                                //
    }                                                                  //
  }                                                                    //
}                                                                      //
                                                                       //
function getActiveGrain(grains) {                                      // 1915
  var idx = activeGrainIndex(grains);                                  // 1916
  return idx == -1 ? undefined : grains[idx];                          // 1917
}                                                                      //
                                                                       //
function activeGrainIndex(grains) {                                    // 1920
  for (var i = 0; i < grains.length; i++) {                            // 1921
    if (grains[i].isActive()) {                                        // 1922
      return i;                                                        // 1923
    }                                                                  //
  }                                                                    //
                                                                       //
  return -1;                                                           // 1927
}                                                                      //
                                                                       //
function grainOriginToIndex(grains, origin) {                          // 1930
  for (var i = 0; i < grains.length; i++) {                            // 1931
    if (grains[i].origin() === origin) {                               // 1932
      return i;                                                        // 1933
    }                                                                  //
  }                                                                    //
                                                                       //
  return -1;                                                           // 1937
}                                                                      //
                                                                       //
function grainIdToIndex(grains, grainId) {                             // 1940
  for (var i = 0; i < grains.length; i++) {                            // 1941
    var grain = grains[i];                                             // 1942
    if (grains[i].grainId() === grainId) {                             // 1943
      return i;                                                        // 1944
    }                                                                  //
  }                                                                    //
                                                                       //
  return -1;                                                           // 1948
}                                                                      //
                                                                       //
function mapGrainStateToTemplateData(grainState) {                     // 1951
  var error = grainState.error();                                      // 1952
  var templateData = {                                                 // 1953
    grainId: grainState.grainId(),                                     // 1954
    active: grainState.isActive(),                                     // 1955
    title: grainState.title(),                                         // 1956
    error: error && error.message,                                     // 1957
    unauthorized: error && error.error == 403,                         // 1958
    notFound: error && error.error == 404,                             // 1959
    appOrigin: grainState.origin(),                                    // 1960
    hasNotLoaded: !grainState.hasLoaded(),                             // 1961
    sessionId: grainState.sessionId(),                                 // 1962
    originalPath: grainState._originalPath,                            // 1963
    interstitial: grainState.shouldShowInterstitial(),                 // 1964
    token: grainState.token(),                                         // 1965
    viewInfo: grainState.viewInfo()                                    // 1966
  };                                                                   //
  return templateData;                                                 // 1968
}                                                                      //
                                                                       //
GrainLog = new Mongo.Collection("grainLog");                           // 1971
// Pseudo-collection created by subscribing to "grainLog", implemented in proxy.js.
                                                                       //
Router.map(function () {                                               // 1974
  this.route("apps", {                                                 // 1975
    path: "/apps",                                                     // 1976
    waitOn: function () {                                              // 1977
      return globalSubs;                                               // 1977
    },                                                                 //
                                                                       //
    data: function () {                                                // 1979
      if (!this.ready()) return;                                       // 1980
      if (!Meteor.userId() && !Meteor.loggingIn()) {                   // 1981
        Router.go("root", {}, { replaceState: true });                 // 1982
      }                                                                //
                                                                       //
      return new SandstormAppList(globalDb, globalQuotaEnforcer);      // 1985
    }                                                                  //
  });                                                                  //
  this.route("newGrainRedirect", {                                     // 1988
    // The path /grain/new used to be where you would go to create new grains.
    // Its functionality has been superceded by the apps route, so redirect in
    // case anyone has the old link saved somewhere.                   //
    path: "/grain/new",                                                // 1992
    onBeforeAction: function () {                                      // 1993
      Router.go("apps", {}, { replaceState: true });                   // 1994
    }                                                                  //
  });                                                                  //
  this.route("grains", {                                               // 1997
    path: "/grain",                                                    // 1998
    waitOn: function () {                                              // 1999
      return globalSubs;                                               // 1999
    },                                                                 //
                                                                       //
    data: function () {                                                // 2001
      if (!this.ready()) return;                                       // 2002
      if (!Meteor.userId() && !Meteor.loggingIn()) {                   // 2003
        Router.go("root", {}, { replaceState: true });                 // 2004
      }                                                                //
                                                                       //
      return new SandstormGrainListPage(globalDb, globalQuotaEnforcer);
    }                                                                  //
  });                                                                  //
  this.route("grain", {                                                // 2010
    path: "/grain/:grainId/:path(.*)?",                                // 2011
    loadingTemplate: "loadingNoMessage",                               // 2012
                                                                       //
    waitOn: function () {                                              // 2014
      return globalSubs;                                               // 2015
    },                                                                 //
                                                                       //
    onBeforeAction: function () {                                      // 2018
      // Only run the hook once.                                       //
      if (this.state.get("beforeActionHookRan")) return this.next();   // 2020
      this.state.set("beforeActionHookRan", true);                     // 2021
      var grainId = this.params.grainId;                               // 2022
      var initialPopup = null;                                         // 2023
      var shareGrain = Session.get("share-grain-" + grainId);          // 2024
      if (shareGrain) {                                                // 2025
        initialPopup = "share";                                        // 2026
      }                                                                //
                                                                       //
      var path = "/" + (this.params.path || "") + (this.originalUrl.match(/[#?].*$/) || "");
      var grains = globalGrains.get();                                 // 2030
      var grainIndex = grainIdToIndex(grains, grainId);                // 2031
      if (grainIndex == -1) {                                          // 2032
        // The element we need to attach our Blaze view to may not exist yet.
        // In that case, defer creating the GrainView until we're sure it's
        // had a chance to render.                                     //
        var openView = (function () {                                  // 2036
          function openView() {                                        // 2036
            var mainContentElement = document.querySelector("body>.main-content");
            if (mainContentElement) {                                  // 2038
              var _grains = globalGrains.get();                        // 2039
              var grainToOpen = new GrainView(grainId, path, undefined, mainContentElement, initialPopup);
              grainToOpen.openSession();                               // 2042
              grainIndex = _grains.push(grainToOpen) - 1;              // 2043
              globalGrains.set(_grains);                               // 2044
              makeGrainIdActive(grainId);                              // 2045
            } else {                                                   //
              Meteor.defer(openView);                                  // 2047
            }                                                          //
          }                                                            //
                                                                       //
          return openView;                                             //
        })();                                                          //
                                                                       //
        openView();                                                    // 2051
      } else {                                                         //
        makeGrainIdActive(grainId);                                    // 2053
      }                                                                //
                                                                       //
      this.next();                                                     // 2056
    },                                                                 //
                                                                       //
    onStop: function () {                                              // 2059
      this.state.set("beforeActionHookRan", undefined);                // 2060
      globalGrains.get().forEach(function (grain) {                    // 2061
        if (grain.isActive()) {                                        // 2062
          grain.setActive(false);                                      // 2063
        }                                                              //
      });                                                              //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("/shared/:token/:path(.*)?", {                            // 2069
    name: "shared",                                                    // 2070
    template: "grain",                                                 // 2071
    loadingTemplate: "loadingNoMessage",                               // 2072
                                                                       //
    waitOn: function () {                                              // 2074
      return [Meteor.subscribe("devPackages"), Meteor.subscribe("tokenInfo", this.params.token), Meteor.subscribe("grainsMenu")];
    },                                                                 //
                                                                       //
    // This subscription gives us the data we need for deciding whether to automatically reveal
    // our identity.                                                   //
    // TODO(soon): Subscribe to contacts instead.                      //
    onBeforeAction: function () {                                      // 2086
      // Run this hook only once. We could accomplish the same thing by using the `onRun()` hook
      // and waiting for `this.ready()`, but for some reason that fails in the case when a user
      // logs in while visiting a /shared/ link.                       //
      if (this.state.get("beforeActionHookRan")) return this.next();   // 2090
      this.state.set("beforeActionHookRan", true);                     // 2091
                                                                       //
      var token = this.params.token;                                   // 2093
      var path = "/" + (this.params.path || "") + (this.originalUrl.match(/[#?].*$/) || "");
      var hash = this.params.hash;                                     // 2095
                                                                       //
      var tokenInfo = TokenInfo.findOne({ _id: token });               // 2097
      if (tokenInfo && tokenInfo.invalidToken) {                       // 2098
        this.state.set("invalidToken", true);                          // 2099
      } else {                                                         //
        (function () {                                                 //
          var grainId = tokenInfo.grainId;                             // 2101
          var grains = globalGrains.get();                             // 2102
          var grainIndex = grainIdToIndex(grains, grainId);            // 2103
          if (grainIndex == -1) {                                      // 2104
            var openView = (function () {                              // 2105
              function openView() {                                    // 2105
                var mainContentElement = document.querySelector("body>.main-content");
                if (mainContentElement) {                              // 2107
                  var _grains2 = globalGrains.get();                   // 2108
                  var grainToOpen = new GrainView(grainId, path, tokenInfo, mainContentElement);
                  grainToOpen.openSession();                           // 2110
                  grainIndex = _grains2.push(grainToOpen) - 1;         // 2111
                  globalGrains.set(_grains2);                          // 2112
                  makeGrainIdActive(grainId);                          // 2113
                } else {                                               //
                  Meteor.defer(openView);                              // 2115
                }                                                      //
              }                                                        //
                                                                       //
              return openView;                                         //
            })();                                                      //
                                                                       //
            openView();                                                // 2119
          } else {                                                     //
            makeGrainIdActive(grainId);                                // 2121
          }                                                            //
        })();                                                          //
      }                                                                //
                                                                       //
      this.next();                                                     // 2125
    },                                                                 //
                                                                       //
    action: function () {                                              // 2128
      if (this.state.get("invalidToken")) {                            // 2129
        this.render("invalidToken", { data: { token: this.params.token } });
      } else {                                                         //
        this.render();                                                 // 2132
      }                                                                //
    },                                                                 //
                                                                       //
    onStop: function () {                                              // 2136
      this.state.set("beforeActionHookRan", undefined);                // 2137
      this.state.set("invalidToken", undefined);                       // 2138
      globalGrains.get().forEach(function (grain) {                    // 2139
        if (grain.isActive()) {                                        // 2140
          grain.setActive(false);                                      // 2141
        }                                                              //
      });                                                              //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("grainLog", {                                             // 2147
    path: "/grainlog/:grainId",                                        // 2148
    layoutTemplate: "noLayout",                                        // 2149
                                                                       //
    waitOn: function () {                                              // 2151
      return [Meteor.subscribe("grainTopBar", this.params.grainId), Meteor.subscribe("grainLog", this.params.grainId)];
    },                                                                 //
                                                                       //
    data: function () {                                                // 2158
      if (this.ready()) {                                              // 2159
        maybeScrollLog();                                              // 2160
        var grain = Grains.findOne(this.params.grainId);               // 2161
        return {                                                       // 2162
          title: grain ? grain.title : "(deleted grain)",              // 2163
          // jscs:disable requireCamelCaseOrUpperCaseIdentifiers       //
          html: AnsiUp.ansi_to_html(GrainLog.find({}, { $sort: { _id: 1 } }).map(function (entry) {
            return entry.text;                                         // 2166
          }).join(""), { use_classes: true })                          //
        };                                                             //
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  // jscs:enable requireCamelCaseOrUpperCaseIdentifiers                //
  this.route("appDetails", {                                           // 2174
    path: "/apps/:appId",                                              // 2175
    template: "appDetails",                                            // 2176
    waitOn: function () {                                              // 2177
      return globalSubs.concat(Meteor.subscribe("appIndex", this.params.appId));
    },                                                                 //
                                                                       //
    data: function () {                                                // 2182
      return new SandstormAppDetails(globalDb, globalQuotaEnforcer, this.params.appId);
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("share", {                                                // 2187
    path: "/share/:grainId/:identityId",                               // 2188
    template: "share",                                                 // 2189
    waitOn: function () {                                              // 2190
      return Meteor.subscribe("grainTopBar", this.params.grainId);     // 2191
    },                                                                 //
                                                                       //
    data: function () {                                                // 2194
      var userId = Meteor.userId();                                    // 2195
      var grainId = this.params.grainId;                               // 2196
      var identityId = this.params.identityId;                         // 2197
      var grain = Grains.findOne({ _id: grainId });                    // 2198
      if (!grain) {                                                    // 2199
        return { grainNotFound: grainId };                             // 2200
      } else {                                                         //
        Session.set("share-grain-" + grainId, identityId);             // 2202
        Router.go("/grain/" + grainId);                                // 2203
      }                                                                //
    }                                                                  //
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=grain.js.map
