(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/install.js                                                   //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
// This file covers /install and /upload.                              //
                                                                       //
var localizedTextPattern = {                                           // 19
  defaultText: String,                                                 // 20
  localizations: Match.Optional([{ locale: String, text: String }])    // 21
};                                                                     //
                                                                       //
var uploadTokens = {};                                                 // 24
                                                                       //
if (Meteor.isServer) {                                                 // 26
  UserActions.allow({                                                  // 27
    insert: function (userId, action) {                                // 28
      // TODO(cleanup): This check keeps breaking. Use a method instead that takes the package
      //   ID as an argument.                                          //
      check(action, {                                                  // 31
        userId: String,                                                // 32
        packageId: String,                                             // 33
        appId: String,                                                 // 34
        appTitle: Match.Optional(localizedTextPattern),                // 35
        appMarketingVersion: Match.Optional(Object),                   // 36
        appVersion: Match.Integer,                                     // 37
        title: localizedTextPattern,                                   // 38
        nounPhrase: Match.Optional(localizedTextPattern),              // 39
        command: {                                                     // 40
          executablePath: Match.Optional(String),                      // 41
          deprecatedExecutablePath: Match.Optional(String),            // 42
          args: Match.Optional([String]),                              // 43
          argv: Match.Optional([String]),                              // 44
          environ: Match.Optional([{ key: String, value: String }])    // 45
        }                                                              //
      });                                                              //
      return userId && isSignedUpOrDemo() && action.userId === userId;
    },                                                                 //
                                                                       //
    remove: function (userId, action) {                                // 51
      return userId && action.userId === userId;                       // 52
    }                                                                  //
  });                                                                  //
                                                                       //
  // Not all users are allowed to upload apps. We need to manually implement authorization
  // because Meteor.userId() is not available in server-side routes.   //
                                                                       //
  Meteor.methods({                                                     // 59
    cancelDownload: function (packageId) {                             // 60
      check(packageId, String);                                        // 61
                                                                       //
      // TODO(security):  Only let user cancel download if they initiated it.
      cancelDownload(packageId);                                       // 64
    },                                                                 //
                                                                       //
    newUploadToken: function () {                                      // 67
      if (!isSignedUp()) {                                             // 68
        throw new Meteor.Error(403, "Unauthorized", "Only invited users can upload apps.");
      }                                                                //
                                                                       //
      if (globalDb.isUninvitedFreeUser()) {                            // 72
        throw new Meteor.Error(403, "Unauthorized", "Only paid users can upload apps.");
      }                                                                //
                                                                       //
      var token = Random.id();                                         // 76
      uploadTokens[token] = setTimeout(function () {                   // 77
        delete uploadTokens[token];                                    // 78
      }, 20 * 60 * 1000);                                              //
                                                                       //
      return token;                                                    // 81
    },                                                                 //
                                                                       //
    upgradeGrains: function (appId, version, packageId) {              // 84
      this.connection.sandstormDb.upgradeGrains(appId, version, packageId, globalBackend);
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
function isSafeDemoAppUrl(url) {                                       // 90
  // For demo accounts, we allow using a bare hash with no URL (which will never upload a new app)
  // and we allow specifying a sandstorm.io URL.                       //
  return !url || url.lastIndexOf("http://sandstorm.io/", 0) === 0 || url.lastIndexOf("https://sandstorm.io/", 0) === 0 || url.lastIndexOf("https://alpha-j7uny7u376jnimcsx34c.sandstorm.io/", 0) === 0 || url.lastIndexOf("https://app-index.sandstorm.io/", 0) === 0;
}                                                                      //
                                                                       //
Meteor.methods({                                                       // 100
  ensureInstalled: function (packageId, url, isRetry) {                // 101
    check(packageId, String);                                          // 102
    check(url, Match.OneOf(String, undefined, null));                  // 103
    check(isRetry, Boolean);                                           // 104
                                                                       //
    if (!packageId.match(/^[a-zA-Z0-9]*$/)) {                          // 106
      throw new Meteor.Error(400, "The package ID contains illegal characters.");
    }                                                                  //
                                                                       //
    if (!this.userId) {                                                // 110
      if (allowDemo && isSafeDemoAppUrl(url)) {                        // 111
        // continue on                                                 //
      } else {                                                         //
          // jscs:ignore disallowEmptyBlocks                           //
          throw new Meteor.Error(403, "You must be logged in to install packages.");
        }                                                              //
    } else if (!isSignedUp() && !isDemoUser()) {                       //
      throw new Meteor.Error(403, "This Sandstorm server requires you to get an invite before installing apps.");
    } else if (isUserOverQuota(Meteor.user())) {                       //
      throw new Meteor.Error(402, "You are out of storage space. Please delete some things and try again.");
    }                                                                  //
                                                                       //
    if (!this.isSimulation) {                                          // 124
      var pkg = Packages.findOne(packageId);                           // 125
                                                                       //
      if (!pkg || pkg.status !== "ready") {                            // 127
        if (!this.userId || isDemoUser() || globalDb.isUninvitedFreeUser()) {
          if (!isSafeDemoAppUrl(url)) {                                // 129
            // TODO(someday): Billing prompt on client side.           //
            throw new Meteor.Error(403, "Sorry, demo and free users cannot upload custom apps; " + "they may only install apps from apps.sandstorm.io.");
          }                                                            //
        }                                                              //
      }                                                                //
                                                                       //
      if (!pkg || isRetry) {                                           // 137
        globalDb.startInstall(packageId, url, isRetry);                // 138
      }                                                                //
    }                                                                  //
  }                                                                    //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 144
  this.route("install", {                                              // 145
    path: "/install/:packageId",                                       // 146
                                                                       //
    waitOn: function () {                                              // 148
      return [Meteor.subscribe("packageInfo", this.params.packageId), Meteor.subscribe("credentials"),
      // We need UserActions and Grains populated so we can check if we need to upgrade them.
      Meteor.subscribe("grainsMenu")];                                 // 153
    },                                                                 //
                                                                       //
    data: function () {                                                // 157
      if (!this.ready()) return;                                       // 158
                                                                       //
      var packageId = this.params.packageId;                           // 160
      var packageUrl = this.params.query && this.params.query.url;     // 161
      var handle = new SandstormAppInstall(packageId, packageUrl, globalDb);
                                                                       //
      var pkg = Packages.findOne(packageId);                           // 164
      if (!Meteor.userId()) {                                          // 165
        if (allowDemo && isSafeDemoAppUrl(packageUrl)) {               // 166
          if (pkg && pkg.status === "ready") {                         // 167
            Router.go("appdemo", { appId: pkg.appId }, { replaceState: true });
            return handle;                                             // 169
          } else {}                                                    //
          // continue on and install...                                //
          // jscs:ignore disallowEmptyBlocks                           //
        } else {                                                       // 172
            handle.setError("You must sign in to install packages.");  // 174
            return handle;                                             // 175
          }                                                            //
      } else {                                                         //
        try {                                                          // 178
          // When the user clicks to install an app, the app store is opened in a new tab. When they
          // choose an app in the app store, they are redirected back to Sandstorm. But we'd really
          // like to bring them back to the tab where they clicked "install apps". It turns out we
          // can exploit a terrible feature of the web platform to do this: window.opener is a
          // pointer to the tab which opened this tab, and we can actually reach right into it and
          // call functions in it. So, we redirect the original tab to install the app, then close
          // this one.                                                 //
          if (globalGrains.get().length === 0 && window.opener.location.hostname === window.location.hostname && window.opener.Router) {
            window.opener.Router.go("install", { packageId: packageId }, { query: this.params.query });
            window.close();                                            // 190
            return handle;                                             // 191
          }                                                            //
        } catch (err) {                                                //
          // Probably security error because window.opener is in a different domain.
        }                                                              //
                                                                       //
        if (!isSignedUp() && !isDemoUser()) {                          // 197
          handle.setError("This Sandstorm server requires you to get an invite before installing apps.");
          return handle;                                               // 199
        }                                                              //
      }                                                                //
                                                                       //
      Meteor.call("ensureInstalled", packageId, packageUrl, false, function (err, result) {
        if (err) {                                                     // 205
          console.log(err);                                            // 206
          handle.setError(err.message);                                // 207
        }                                                              //
      });                                                              //
                                                                       //
      if (pkg === undefined) {                                         // 211
        if (!packageUrl) {                                             // 212
          handle.setError("Unknown package ID: " + packageId + "\nPerhaps it hasn't been uploaded?");
        }                                                              //
                                                                       //
        return handle;                                                 // 217
      }                                                                //
                                                                       //
      if (pkg.status !== "ready") {                                    // 220
        if (pkg.status === "failed") {                                 // 221
          handle.setError(pkg.error);                                  // 222
        }                                                              //
                                                                       //
        return handle;                                                 // 225
      }                                                                //
                                                                       //
      // From here on, we know the package is installed in the global store, but we
      // might have some per-user things to attend to.                 //
      if (handle.isInstalled()) {                                      // 230
        // The app is installed, so let's send the user to it!  We use `replaceState` so that if
        // the user clicks "back" they don't just get redirected forward again, but end up back
        // at the app list.                                            //
        Router.go("appDetails", { appId: handle.appId() }, { replaceState: true });
      }                                                                //
                                                                       //
      return handle;                                                   // 237
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("upload", {                                               // 241
    where: "server",                                                   // 242
    path: "/upload/:token",                                            // 243
                                                                       //
    action: function () {                                              // 245
      if (!this.params.token || !uploadTokens[this.params.token]) {    // 246
        this.response.writeHead(403, {                                 // 247
          "Content-Type": "text/plain"                                 // 248
        });                                                            //
        this.response.write("Invalid upload token.");                  // 250
        this.response.end();                                           // 251
      } else if (this.request.method === "POST") {                     //
        try {                                                          // 253
          var packageId = promiseToFuture(doClientUpload(this.request)).wait();
          this.response.writeHead(200, {                               // 255
            "Content-Length": packageId.length,                        // 256
            "Content-Type": "text/plain"                               // 257
          });                                                          //
          this.response.write(packageId);                              // 259
          this.response.end();                                         // 260
          clearTimeout(uploadTokens[this.params.token]);               // 261
          delete uploadTokens[this.params.token];                      // 262
        } catch (error) {                                              //
          console.error(error.stack);                                  // 264
          this.response.writeHead(500, {                               // 265
            "Content-Type": "text/plain"                               // 266
          });                                                          //
          this.response.write("Unpacking SPK failed; is it valid?");   // 268
          this.response.end();                                         // 269
        };                                                             //
      } else {                                                         //
        this.response.writeHead(405, {                                 // 272
          "Content-Type": "text/plain"                                 // 273
        });                                                            //
        this.response.write("You can only POST here.");                // 275
        this.response.end();                                           // 276
      }                                                                //
    }                                                                  //
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=install.js.map
