(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var ECMAScript = Package.ecmascript.ECMAScript;
var _ = Package.underscore._;
var Random = Package.random.Random;
var SandstormDb = Package['sandstorm-db'].SandstormDb;
var SandstormBackend = Package['sandstorm-backend'].SandstormBackend;
var MongoInternals = Package.mongo.MongoInternals;
var Mongo = Package.mongo.Mongo;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var check = Package.check.check;
var Match = Package.check.Match;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;

(function(){

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                  //
// packages/accounts-identity/accounts-identity-server.js                                                           //
//                                                                                                                  //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                    //
// Sandstorm - Personal Cloud Sandbox                                                                               //
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                            //
// All rights reserved.                                                                                             //
//                                                                                                                  //
// Licensed under the Apache License, Version 2.0 (the "License");                                                  //
// you may not use this file except in compliance with the License.                                                 //
// You may obtain a copy of the License at                                                                          //
//                                                                                                                  //
//   http://www.apache.org/licenses/LICENSE-2.0                                                                     //
//                                                                                                                  //
// Unless required by applicable law or agreed to in writing, software                                              //
// distributed under the License is distributed on an "AS IS" BASIS,                                                //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                         //
// See the License for the specific language governing permissions and                                              //
// limitations under the License.                                                                                   //
                                                                                                                    //
function linkIdentityToAccountInternal(db, backend, identityId, accountId) {                                        // 17
  // Links the identity to the account. If the account is a demo account, makes the account durable                 //
  // and gives the identity login access to it.                                                                     //
                                                                                                                    //
  check(db, SandstormDb);                                                                                           // 21
  check(backend, SandstormBackend);                                                                                 // 22
  check(identityId, String);                                                                                        // 23
  check(accountId, String);                                                                                         // 24
                                                                                                                    //
  var accountUser = Meteor.users.findOne({ _id: accountId });                                                       // 26
  if (!accountUser) {                                                                                               // 27
    throw new Meteor.Error(404, "No account found with ID " + accountId);                                           // 28
  }                                                                                                                 //
                                                                                                                    //
  if (accountUser.profile) {                                                                                        // 31
    throw new Meteor.Error(400, "Cannot link an identity to another identity.");                                    // 32
  }                                                                                                                 //
                                                                                                                    //
  if (!!_.findWhere(accountUser.loginIdentities, { id: identityId }) || !!_.findWhere(accountUser.nonloginIdentities, { id: identityId })) {
    throw new Meteor.Error(400, "Cannot link an identity that's alread linked to this account.");                   // 37
  }                                                                                                                 //
                                                                                                                    //
  var identityUser = Meteor.users.findOne({ _id: identityId });                                                     // 40
                                                                                                                    //
  if (!identityUser) {                                                                                              // 42
    throw new Meteor.Error(404, "No identity found with ID " + identityId);                                         // 43
  }                                                                                                                 //
                                                                                                                    //
  if (!identityUser.profile) {                                                                                      // 46
    throw new Meteor.Error(400, "Cannot link an account to another account");                                       // 47
  }                                                                                                                 //
                                                                                                                    //
  db.deleteUnusedAccount(backend, identityUser._id);                                                                // 50
  if (Meteor.users.findOne({ "loginIdentities.id": identityUser._id })) {                                           // 51
    throw new Meteor.Error(403, "Cannot link an identity that can already log into another account");               // 52
  }                                                                                                                 //
                                                                                                                    //
  var modifier = undefined;                                                                                         // 56
  if (accountUser.expires) {                                                                                        // 57
    if (Meteor.users.findOne({ "nonloginIdentities.id": identityUser._id })) {                                      // 58
      throw new Meteor.Error(403, "Cannot create an account for an identity that's " + "already linked to another account.");
    }                                                                                                               //
                                                                                                                    //
    modifier = { $push: { loginIdentities: { id: identityUser._id } },                                              // 63
      $unset: { expires: 1 },                                                                                       // 64
      $set: { upgradedFromDemo: Date.now() } };                                                                     // 65
    if (Meteor.settings["public"].quotaEnabled) {                                                                   // 66
      // Demo users never got the referral notification. Send it now:                                               //
      db.sendReferralProgramNotification(accountUser._id);                                                          // 68
    }                                                                                                               //
  } else {                                                                                                          //
    modifier = { $push: { nonloginIdentities: { id: identityUser._id } } };                                         // 72
  }                                                                                                                 //
                                                                                                                    //
  // Make sure not to add the same identity twice.                                                                  //
  Meteor.users.update({ _id: accountUser._id, "nonloginIdentities.id": { $ne: identityUser._id },                   // 76
    "loginIdentities.id": { $ne: identityUser._id } }, modifier);                                                   // 77
                                                                                                                    //
  if (accountUser.expires) {                                                                                        // 80
    var demoIdentityId = SandstormDb.getUserIdentityIds(accountUser)[0];                                            // 81
    Meteor.users.update({ _id: demoIdentityId }, { $unset: { expires: 1 },                                          // 82
      $set: { upgradedFromDemo: Date.now() } });                                                                    // 84
                                                                                                                    //
    // Mark the demo identity as nonlogin. It'd be nicer if the identity started out as nonlogin,                   //
    // but to get that to work we would need to adjust the account creation and first login logic.                  //
    Meteor.users.update({ _id: accountUser._id,                                                                     // 88
      "loginIdentities.id": demoIdentityId,                                                                         // 89
      "nonloginIdentities.id": { $not: { $eq: demoIdentityId } } }, { $pull: { loginIdentities: { id: demoIdentityId } },
      $push: { nonloginIdentities: { id: demoIdentityId } } });                                                     // 92
  }                                                                                                                 //
}                                                                                                                   //
                                                                                                                    //
Meteor.methods({                                                                                                    // 97
  loginWithIdentity: function (accountUserId) {                                                                     // 98
    // Logs into the account with ID `accountUserId`. Throws an exception if the current user is                    //
    // not an identity user listed in the account's `loginIdentities` field. This method is not                     //
    // intended to be called directly; client-side code should only invoke it through                               //
    // `Meteor.loginWithIdentity()`, which additionally maintains the standard Meteor client-side                   //
    // login state.                                                                                                 //
                                                                                                                    //
    check(accountUserId, String);                                                                                   // 105
                                                                                                                    //
    var identityUser = Meteor.user();                                                                               // 107
    if (!identityUser || !identityUser.profile) {                                                                   // 108
      throw new Meteor.Error(403, "Must be already logged in as an identity.");                                     // 109
    }                                                                                                               //
                                                                                                                    //
    var accountUser = Meteor.users.findOne(accountUserId);                                                          // 112
    if (!accountUser) {                                                                                             // 113
      throw new Meteor.Error(404, "No such user found: " + accountUserId);                                          // 114
    }                                                                                                               //
                                                                                                                    //
    var linkedIdentity = _.findWhere(accountUser.loginIdentities, { id: identityUser._id });                        // 117
                                                                                                                    //
    if (!linkedIdentity) {                                                                                          // 119
      throw new Meteor.Error(403, "Current identity is not a login identity for account " + accountUserId);         // 120
    }                                                                                                               //
                                                                                                                    //
    return Accounts._loginMethod(this, "loginWithIdentity", [accountUserId], "identity", function () {              // 124
      return { userId: accountUserId };                                                                             // 125
    });                                                                                                             //
  },                                                                                                                //
                                                                                                                    //
  createAccountForIdentity: function () {                                                                           // 128
    // Creates a new account for the currently-logged-in identity.                                                  //
                                                                                                                    //
    var user = Meteor.user();                                                                                       // 131
    if (!(user && user.profile)) {                                                                                  // 132
      throw new Meteor.Error(403, "Must be logged in as an identity in order to create an account.");               // 133
    }                                                                                                               //
                                                                                                                    //
    if (Meteor.users.findOne({                                                                                      // 136
      $or: [{ "loginIdentities.id": user._id }, { "nonloginIdentities.id": user._id }]                              // 137
    })) {                                                                                                           //
      throw new Meteor.Error(403, "Cannot create an account for an identity that's already " + "linked to another account.");
    }                                                                                                               //
                                                                                                                    //
    var newUser = {                                                                                                 // 146
      loginIdentities: [{ id: user._id }],                                                                          // 147
      nonloginIdentities: []                                                                                        // 148
    };                                                                                                              //
    if (user.services.dev) {                                                                                        // 150
      newUser.signupKey = "devAccounts";                                                                            // 151
      if (user.services.dev.isAdmin) {                                                                              // 152
        newUser.isAdmin = true;                                                                                     // 153
      }                                                                                                             //
                                                                                                                    //
      if (user.services.dev.hasCompletedSignup) {                                                                   // 156
        newUser.hasCompletedSignup = true;                                                                          // 157
      }                                                                                                             //
    } else if (user.expires) {                                                                                      //
      // Demo user.                                                                                                 //
      newUser.expires = user.expires;                                                                               // 161
      if (!!user.appDemoId) {                                                                                       // 162
        newUser.appDemoId = user.appDemoId;                                                                         // 163
      }                                                                                                             //
    }                                                                                                               //
                                                                                                                    //
    var options = {};                                                                                               // 167
                                                                                                                    //
    // This will throw an error if the identity has been added as a login identity to some                          //
    // other account while we were executing the body of this method.                                               //
    return Accounts.insertUserDoc(options, newUser);                                                                // 171
  },                                                                                                                //
                                                                                                                    //
  linkIdentityToAccount: function (token) {                                                                         // 174
    // Links the identity of the current user to the account that has `token` as a resume token.                    //
    // If the account is a demo account, makes the account durable and gives the identity login                     //
    // access to it.                                                                                                //
                                                                                                                    //
    check(token, String);                                                                                           // 179
                                                                                                                    //
    if (!this.userId) {                                                                                             // 181
      throw new Meteor.Error(403, "Cannot link to account if not logged in.");                                      // 182
    }                                                                                                               //
                                                                                                                    //
    var hashed = Accounts._hashLoginToken(token);                                                                   // 185
    var accountUser = Meteor.users.findOne({ "services.resume.loginTokens.hashedToken": hashed });                  // 186
                                                                                                                    //
    linkIdentityToAccountInternal(this.connection.sandstormDb, this.connection.sandstormBackend, this.userId, accountUser._id);
  },                                                                                                                //
                                                                                                                    //
  unlinkIdentity: function (accountUserId, identityId) {                                                            // 192
    // Unlinks the identity with ID `identityId` from the account with ID `accountUserId`.                          //
                                                                                                                    //
    check(identityId, String);                                                                                      // 195
    check(accountUserId, String);                                                                                   // 196
                                                                                                                    //
    if (!this.userId) {                                                                                             // 198
      throw new Meteor.Error(403, "Not logged in.");                                                                // 199
    }                                                                                                               //
                                                                                                                    //
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                                    // 202
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                              // 203
    }                                                                                                               //
                                                                                                                    //
    var identityUser = Meteor.users.findOne({ _id: identityId });                                                   // 206
    Meteor.users.update({ _id: accountUserId }, { $pull: { nonloginIdentities: { id: identityId },                  // 207
        loginIdentities: { id: identityId } } });                                                                   // 209
  },                                                                                                                //
                                                                                                                    //
  setIdentityAllowsLogin: function (identityId, allowLogin) {                                                       // 212
    // Sets whether the current account allows the identity with ID `identityId` to log in.                         //
                                                                                                                    //
    check(identityId, String);                                                                                      // 215
    check(allowLogin, Boolean);                                                                                     // 216
    if (!this.userId) {                                                                                             // 217
      throw new Meteor.Error(403, "Not logged in.");                                                                // 218
    }                                                                                                               //
                                                                                                                    //
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                                    // 221
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                              // 222
    }                                                                                                               //
                                                                                                                    //
    if (allowLogin) {                                                                                               // 225
      Meteor.users.update({ _id: this.userId,                                                                       // 226
        "nonloginIdentities.id": identityId,                                                                        // 227
        "loginIdentities.id": { $not: { $eq: identityId } } }, { $pull: { nonloginIdentities: { id: identityId } },
        $push: { loginIdentities: { id: identityId } } });                                                          // 230
    } else {                                                                                                        //
      Meteor.users.update({ _id: this.userId,                                                                       // 232
        "loginIdentities.id": identityId,                                                                           // 233
        "nonloginIdentities.id": { $not: { $eq: identityId } } }, { $pull: { loginIdentities: { id: identityId } },
        $push: { nonloginIdentities: { id: identityId } } });                                                       // 236
    }                                                                                                               //
  },                                                                                                                //
                                                                                                                    //
  logoutIdentitiesOfCurrentAccount: function () {                                                                   // 240
    // Logs out all identities that are allowed to log in to the current account.                                   //
    var user = Meteor.user();                                                                                       // 242
    if (user && user.loginIdentities) {                                                                             // 243
      user.loginIdentities.forEach(function (identity) {                                                            // 244
        Meteor.users.update({ _id: identity.id }, { $set: { "services.resume.loginTokens": [] } });                 // 245
      });                                                                                                           //
    }                                                                                                               //
  }                                                                                                                 //
});                                                                                                                 //
                                                                                                                    //
Accounts.linkIdentityToAccount = function (db, backend, identityId, accountId) {                                    // 251
  // Links the identity to the account. If the account is a demo account, makes it durable and                      //
  // gives the identity login access to it.                                                                         //
  check(db, SandstormDb);                                                                                           // 254
  check(backend, SandstormBackend);                                                                                 // 255
  check(identityId, String);                                                                                        // 256
  check(accountId, String);                                                                                         // 257
  linkIdentityToAccountInternal(db, backend, identityId, accountId);                                                // 258
};                                                                                                                  //
                                                                                                                    //
Meteor.publish("accountsOfIdentity", function (identityId) {                                                        // 261
  check(identityId, String);                                                                                        // 262
  if (!SandstormDb.ensureSubscriberHasIdentity(this, identityId)) return;                                           // 263
                                                                                                                    //
  // We maintain a map from identity IDs to live query handles that track profile changes.                          //
  var loginIdentities = {};                                                                                         // 266
                                                                                                                    //
  var _this = this;                                                                                                 // 268
  function addIdentitiesOfAccount(account) {                                                                        // 269
    account.loginIdentities.forEach(function (identity) {                                                           // 270
      if (!(identity.id in loginIdentities)) {                                                                      // 271
        var user = Meteor.users.findOne({ _id: identity.id });                                                      // 272
        if (user) {                                                                                                 // 273
          SandstormDb.fillInProfileDefaults(user);                                                                  // 274
          SandstormDb.fillInIntrinsicName(user);                                                                    // 275
          var filteredUser = _.pick(user, "_id", "profile");                                                        // 276
          filteredUser.loginAccountId = account._id;                                                                // 277
          filteredUser.sourceIdentityId = identityId;                                                               // 278
          _this.added("loginIdentitiesOfLinkedAccounts", user._id, filteredUser);                                   // 279
        }                                                                                                           //
                                                                                                                    //
        loginIdentities[identity.id] = Meteor.users.find({ _id: identity.id }, { fields: { profile: 1 } }).observeChanges({
          changed: function (id, fields) {                                                                          // 284
            _this.changed("loginIdentitiesOfLinkedAccounts", id, fields);                                           // 285
          }                                                                                                         //
        });                                                                                                         //
      }                                                                                                             //
    });                                                                                                             //
  }                                                                                                                 //
                                                                                                                    //
  var cursor = Meteor.users.find({                                                                                  // 292
    $or: [{ "loginIdentities.id": identityId }, { "nonloginIdentities.id": identityId }]                            // 293
  });                                                                                                               //
                                                                                                                    //
  var handle = cursor.observe({                                                                                     // 299
    added: function (account) {                                                                                     // 300
      addIdentitiesOfAccount(account);                                                                              // 301
    },                                                                                                              //
                                                                                                                    //
    changed: function (newAccount, oldAccount) {                                                                    // 304
      addIdentitiesOfAccount(newAccount);                                                                           // 305
    },                                                                                                              //
                                                                                                                    //
    removed: function (account) {                                                                                   // 308
      account.loginIdentities.forEach(function (identity) {                                                         // 309
        if (identity.id in loginIdentities) {                                                                       // 310
          self.removed("loginIdentitiesOfLinkedAccounts", identity.id);                                             // 311
          loginIdentities[identity.id].stop();                                                                      // 312
          delete loginIdentities[identity.id];                                                                      // 313
        }                                                                                                           //
      });                                                                                                           //
    }                                                                                                               //
  });                                                                                                               //
  this.ready();                                                                                                     // 318
                                                                                                                    //
  this.onStop(function () {                                                                                         // 320
    handle.stop();                                                                                                  // 321
    Object.keys(loginIdentities).forEach(function (identityId) {                                                    // 322
      loginIdentities[identityId].stop();                                                                           // 323
      delete loginIdentities[identityId];                                                                           // 324
    });                                                                                                             //
  });                                                                                                               //
});                                                                                                                 //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['accounts-identity'] = {};

})();

//# sourceMappingURL=accounts-identity.js.map
